#! -*- coding: utf-8
import os
import os.path as path
from glob import glob
from concurrent.futures import ProcessPoolExecutor

import numpy as np
import pandas as pd


def __proc__(datafile):
    domain, _ = path.splitext(path.basename(datafile))
    data = np.loadtxt(datafile, delimiter="\t")
    categories = np.unique(data[:, 0])
    rets = []
    for category in categories:
        indexs = np.where(data[:, 0] == category)[0]
        d = data[indexs]
        shape = d.shape
        nan_mask = np.isnan(d)
        seqs = np.argmax(
            ~np.isnan(d) * np.arange(shape[1]).reshape(1, -1), axis=1)+1
        rets.append([domain, category, shape[0],
                     np.sum(np.isnan(np.sum(data, axis=1))), shape[1],
                     np.max(seqs), np.min(seqs)])
    return rets

    # data = data[:, 1:]  # 先頭データ(カテゴリ)を削除
    # shape = data.shape
    # nan_mask = np.isnan(data)
    # seqs = np.argmax(
    #     ~np.isnan(data) * np.arange(shape[1]).reshape(1, -1), axis=1)+1
    # return domain, shape[0], np.sum(np.isnan(np.sum(data, axis=1))), shape[1], np.max(seqs), np.min(seqs)
if __name__ == "__main__":
    datadir = "datas"
    datainfo = "%s/datainfo_categories.csv" % datadir

    executor = ProcessPoolExecutor()
    tasks = []
    for datafile in glob(path.join(datadir, "*.tsv")):
        tasks.append(executor.submit(__proc__, datafile))
    headers = ["domain", "category", "count",
               "nan_count", "sequence", "max_seq", "min_seq"]
    with open(datainfo, "wt") as f:
        f.write(",".join(headers))
        f.write("\n")
        for task in tasks:
            ret = task.result()
            for line in ret:
                f.write(",".join(map(str, line)))
                f.write("\n")
            # f.write(",".join(map(str, task.result())))
            # f.write("\n")
    df = pd.read_csv(datainfo)
    df.sort_values(by=["domain"], inplace=True)
    df.to_csv(datainfo, index=False)
