#! -*- coding: utf-8
import os
import os.path as path
from glob import glob
import pandas as pd
import numpy as np

from argparse import ArgumentParser

if __name__ == "__main__":
    parser = ArgumentParser()
    parser.add_argument("rootdir", type=str)
    parser.add_argument("--delimiter", type=str, default=",")
    args = parser.parse_args()

    # target_methods = ["attention", "bidirectional", "meta_learning",
    #                   "linear", "nonlinear", "cross_domain",
    #                   "maml_linear", "maml_nonlinear", "maml",
    #                   "domain_specific_linear", "domain_specific_nonlinear", "domain_specific",
    #                   "diff_previous"]
    # target_methods = ["attention",
    #                   "linear", "nonlinear", "cross_domain",
    #                   "maml_linear", "maml_nonlinear", "maml",
    #                   "domain_specific_linear", "domain_specific_nonlinear", "domain_specific",
    #                   # "diff_previous"
    #                   ]
    # target_methods = ["attention", "attention_only", "bidirectional", "meta_learning",
    #                   "linear", "nonlinear", "cross_domain",
    #                   "maml_linear", "maml_nonlinear", "maml",
    #                   "domain_specific_linear", "domain_specific_nonlinear", "domain_specific",
    #                   # "diff_previous"
    #                   ]
    target_methods = ["attention", "maml", "cross_domain",
                      "meta_learning", "bidirectional", "attention_only",
                      "diff_previous"]
    methodset = set()
    cols = ["domain"]

    score = []
    for f in glob(path.join(args.rootdir, "dataset*", "score.csv")):
        dataset = path.basename(path.dirname(f))
        df = pd.read_csv(f, sep=args.delimiter)
        # filter target
        # df = df.loc[df["type"] == "target", targets].copy()
        df = df.loc[df["type"] == "target"].copy()
        methodset |= set(df.columns)
        score.append(df)
    score = pd.concat(score)
    score = score.replace(".query", {"domain": ""},
                          regex=True)

    # 実験対象の手法抽出
    methods = []
    methodset.remove("domain")
    for method in target_methods:
        if not method in methodset:
            continue
        methods.append(method)
        cols.append(method)
    score = score.loc[:, cols].copy()
    score.loc[:, cols[1:]] = np.sqrt(score.loc[:, cols[1:]])

    # score.drop(labels="category", axis=1)
    groups = score.groupby(["domain"])
    means = groups.mean().reset_index().sort_values("domain")
    sigmas = groups.var().reset_index().sort_values("domain")
    counts = groups.count().reset_index().sort_values("domain")
    means.to_csv(path.join(args.rootdir, "domain_means.csv"),
                 index=False)
    sigmas.to_csv(path.join(args.rootdir, "domain_sigmas.csv"),
                  index=False)

    best_idxs = np.argmin(means.iloc[:, 1:].values, axis=1)
    print(best_idxs)
    best_counts = {c: 0 for c in methods}
    for idx in best_idxs:
        method = methods[idx]
        best_counts[method] += 1
    print(best_counts)

    with open(path.join(args.rootdir, "counts.csv"), "wt") as f:
        f.write(",".join(["method", "count"]))
        f.write("\n")
        for col in cols[1:]:
            f.write("%s,%d\n" % (col, best_counts[col]))

    with open(path.join(args.rootdir, "counts.tex"), "wt") as f:
        f.write("\\begin{table}[htb]\n")
        indent = "    "
        f.write("%s\\begin{tabular}{lr} \\hline\n" % indent)
        indent += "    "
        f.write(indent)
        f.write(" & ".join(["", "count"]))
        f.write(" \\\\ \n%s\\hline\n" % indent)

        for col in cols[1:]:
            f.write(indent)
            f.write("%s%s & %d \\\\ \n" % (indent, col, best_counts[col]))
        f.write("%s\\hline\n" % (indent))
        indent = indent[-4:]
        f.write("    \\end{tabular}\n")
        f.write("\\end{table}\n")

    with open(path.join(args.rootdir, "score_domain.csv"), "wt") as f:
        f.write(cols[0])
        for col in cols[1:]:
            f.write(",%s,%s_stderr" % (col, col))
        f.write("\n")

        for (_, mean), (_, sigma), (_, count) in zip(means.iterrows(), sigmas.iterrows(), counts.iterrows()):
            domain = mean.domain.replace(".query", "")
            line = [domain]
            for col in cols[1:]:
                # print(col, mean[col], sigma[col], count[col])
                line.append(mean[col])
                stderr = np.sqrt(sigma[col]/count[col])  # 標準誤差
                line.append(stderr)

            f.write(",".join(map(str, line)))
            f.write("\n")

    with open(path.join(args.rootdir, "score_domain_with_stderr.tex"), "wt") as f:
        f.write("\\begin{table}[htb]\n")
        indent = "    "
        f.write("%s\\begin{tabular}{l" % indent)
        f.write("".join(["r"] * (len(cols) - 1)))
        f.write("} \\hline\n")
        indent += "    "
        header = [""]
        header.extend(cols[1:])
        f.write(indent)
        f.write(" & ".join(header))
        f.write(" \\\\ \n%s\\hline\n" % indent)

        for (_, mean), (_, sigma), (_, count), best_idx in zip(means.iterrows(), sigmas.iterrows(), counts.iterrows(), best_idxs):
            f.write(indent)
            f.write(mean.domain.replace(".query", ""))

            for i, col in enumerate(cols[1:]):
                m, s, c = mean[col], sigma[col], count[col]
                if i == best_idx:
                    f.write(" & \\(\\textbf{%0.04f \\pm %0.04f}\\)" % (
                        m, np.sqrt(s/c)))
                else:
                    f.write(" & \\(%0.04f \\pm %0.04f \\)" % (m, np.sqrt(s/c)))
            f.write(" \\\\ \n")
        f.write("%s\\hline\n" % (indent))

        mean = means.mean()
        sigma = sigmas.var()
        count = counts.count()

        averages, stderrs, bests = [], [], []
        for col in cols[1:]:
            m, s, c = mean[col], sigma[col], count[col]
            averages.append(m)
            stderrs.append(np.sqrt(s/c))
            bests.append(best_counts[col])
        best_average_idx = np.argmin(averages)

        f.write("%sAverage" % indent)
        for i, col in enumerate(cols[1:]):
            m, s = averages[i], stderrs[i]
            if i == best_average_idx:
                f.write(" & \\(\\textbf{%0.04f \\pm %0.04f}\\)" % (m, s))
            else:
                f.write(" & \\(%0.04f \\pm %0.04f \\)" % (m, s))

        best_count_idx = np.argmax(bests)
        f.write("%s\\hline\n" % (indent))
        f.write("%sCount" % indent)
        for i, col in enumerate(cols[1:]):
            c = best_counts[col]
            if i == best_count_idx:
                f.write(" & \\(\\textbf{%d}\\)" % (c))
            else:
                f.write(" & \\(%d\\)" % (c))
        indent = indent[-4:]
        f.write("    \\end{tabular}\n")
        f.write("\\end{table}\n")

    with open(path.join(args.rootdir, "score_domain.tex"), "wt") as f:
        f.write("\\begin{table}[htb]\n")
        indent = "    "
        f.write("%s\\begin{tabular}{l" % indent)
        f.write("".join(["r"] * (len(cols) - 1)))
        f.write("} \\hline\n")
        indent += "    "
        header = [""]
        header.extend(cols[1:])
        f.write(indent)
        f.write(" & ".join(header))
        f.write(" \\\\ \n%s\\hline\n" % indent)

        for (_, mean), (_, sigma), (_, count), best_idx in zip(means.iterrows(), sigmas.iterrows(), counts.iterrows(), best_idxs):
            f.write(indent)
            f.write(mean.domain.replace(".query", ""))

            for i, col in enumerate(cols[1:]):
                m, s, c = mean[col], sigma[col], count[col]
                if i == best_idx:
                    f.write(" & \\textbf{%0.04f}" % (m))
                else:
                    f.write(" & %0.04f" % (m))
            f.write(" \\\\ \n")
        f.write("%s\\hline\n" % (indent))

        mean = means.mean()
        sigma = sigmas.var()
        count = counts.count()

        averages, stderrs, bests = [], [], []
        for col in cols[1:]:
            m, s, c = mean[col], sigma[col], count[col]
            averages.append(m)
            stderrs.append(np.sqrt(s/c))
            bests.append(best_counts[col])
        best_average_idx = np.argmin(averages)

        f.write("%sAverage" % indent)
        for i, col in enumerate(cols[1:]):
            m, s = averages[i], stderrs[i]
            if i == best_average_idx:
                f.write(" & \\textbf{%0.04f}" % (m))
            else:
                f.write(" & %0.04f" % (m))

        best_count_idx = np.argmax(bests)
        f.write("%s\\hline\n" % (indent))
        f.write("%sCount" % indent)
        for i, col in enumerate(cols[1:]):
            c = best_counts[col]
            if i == best_count_idx:
                f.write(" & \\(\\textbf{%d}\\)" % (c))
            else:
                f.write(" & \\(%d\\)" % (c))
        indent = indent[-4:]
        f.write("    \\end{tabular}\n")
        f.write("\\end{table}\n")
