#! -*- coding: utf-8
import os
import os.path as path
from argparse import ArgumentParser
from concurrent.futures import ProcessPoolExecutor
from glob import iglob as glob

import numpy as np
import pandas as pd

if os.name == "posix":
    import matplotlib
    matplotlib.use("Agg")
    from matplotlib import pyplot as plt
    from matplotlib.backends.backend_pdf import PdfPages
else:
    from matplotlib import pyplot as plt
    from matplotlib.backends.backend_pdf import PdfPages

if __name__ == "__main__":
    parser = ArgumentParser()
    parser.add_argument("rootdir", type=str)
    parser.add_argument("--methods",
                        type=str, nargs="+",
                        default=["attention", "maml"])
    parser.add_argument("--delimiter", type=str, default="\t")
    args = parser.parse_args()

    pdf = path.join(args.rootdir, "losses.pdf")
    losses = {}
    for method in args.methods:
        losses[method] = []
        for log_file in glob(path.join(args.rootdir, "dataset*", method, "train.log")):
            log = pd.read_csv(log_file, sep=args.delimiter)
            log["dataset"] = path.basename(
                path.dirname(path.dirname(log_file)))
            losses[method].append(log)

    cmap = plt.cm.tab20
    phases = ["train", "validation", "target"]
    linewidth = 1.0
    elinewidth = 0.3
    with PdfPages(pdf) as pdf:
        # average and var
        counts = {method: len(losses[method]) for method in args.methods}
        groups = {method: pd.concat(losses[method]).drop(
            "dataset", axis=1).groupby("epoch") for method in args.methods}
        # 各手法毎にデータセット間の平均二乗誤差の平均、標準誤差を算出
        datas = {method: [groups[method].mean(), groups[method].std() / np.sqrt(counts[method])]
                 for method in args.methods}
        c_idx = 0
        for phase in phases:
            for method in args.methods:
                c = cmap(c_idx)
                d = datas[method]
                x = d[0].index
                y = d[0].loc[:, "%s_loss" % phase]
                yerr = d[1].loc[:, "%s_loss" % phase]
                plt.errorbar(x, y, yerr=yerr, color=c, ecolor=c,
                             linewidth=linewidth, elinewidth=elinewidth,
                             label="%s %s" % (phase, method))
                c_idx += 1
        plt.legend()
        plt.title("average loss")
        pdf.savefig()
        plt.close()
        for idx in range(counts[args.methods[0]]):
            datas = {method: losses[method][idx] for method in args.methods}
            c_idx = 0
            for phase in phases:
                for method in args.methods:
                    c = cmap(c_idx)
                    d = datas[method]
                    x = d.index
                    y = d.loc[:, "%s_loss" % phase]
                    plt.plot(x, y, color=c, linewidth=linewidth,
                             label="%s %s" % (phase, method))
                    c_idx += 1
            plt.legend()
            plt.title("dataset%02d loss" % idx)
            pdf.savefig()
            plt.close()

        pass
