#! -*- coding: utf-8
import os
import os.path as path
from argparse import ArgumentParser
from concurrent.futures import ProcessPoolExecutor
from glob import iglob as glob

import numpy as np
import pandas as pd

if os.name == "posix":
    import matplotlib
    matplotlib.use("Agg")
    from matplotlib import pyplot as plt
    from matplotlib.backends.backend_pdf import PdfPages
else:
    from matplotlib import pyplot as plt
    from matplotlib.backends.backend_pdf import PdfPages

if __name__ == "__main__":
    parser = ArgumentParser()
    parser.add_argument("rootdir", type=str)
    parser.add_argument("--epochs", default=300, type=int)
    parser.add_argument("--delimiter", type=str, default="\t")
    args = parser.parse_args()

    methods = ["domain_specific",
               "domain_specific_linear",
               "domain_specific_nonlinear",
               "meta_learning", "bidirectional", "attention",
               "cross_domain", "linear", "nonlinear",
               "maml", "maml_linear", "maml_nonlinear",
               ]

    pdf = path.join(args.rootdir, "losses.pdf")
    losses = {}
    for method in methods:
        losses[method] = []
        for log_file in glob(path.join(args.rootdir, "%s_*.log" % (method))):
            try:
                log = pd.read_csv(log_file, sep=args.delimiter)
                if not len(log) == args.epochs:
                    continue
                log["dataset"] = path.basename(
                    path.dirname(path.dirname(log_file)))
                losses[method].append(log)
            except:
                continue

    # cmap = plt.cm.tab20
    cmap = plt.cm.tab10
    phases = ["train", "validation", "target"]
    linewidth = 1.0
    elinewidth = 0.3
    with PdfPages(pdf) as pdf:
        # average and var
        counts, datas = {}, {}
        for method in methods:
            count = len(losses[method])
            if count == 0:
                continue
            group = pd.concat(losses[method]).drop(
                "dataset", axis=1).groupby("epoch")
            datas[method] = [group.mean(), group.std() / np.sqrt(count)]
            counts[method] = count
        for method in counts.keys():
            c_idx = 0
            for phase in phases:
                c = cmap(c_idx)
                d = datas[method]
                x = d[0].index
                y = d[0].loc[:, "%s_loss" % phase]
                yerr = d[1].loc[:, "%s_loss" % phase]
                plt.errorbar(x, y, yerr=yerr, color=c, ecolor=c,
                             linewidth=linewidth, elinewidth=elinewidth,
                             label="%s" % (phase))
                c_idx += 1
            plt.ylim(0.0, 0.2)
            plt.legend()
            plt.title("%s: average loss" % method)
            pdf.savefig()
            plt.close()

        # for idx in range(counts[methods[0]]):
        #     datas = {method: losses[method][idx] for method in methods}
        #     c_idx = 0
        #     for phase in phases:
        #         for method in methods:
        #             c = cmap(c_idx)
        #             d = datas[method]
        #             x = d.index
        #             y = d.loc[:, "%s_loss" % phase]
        #             plt.plot(x, y, color=c, linewidth=linewidth,
        #                      label="%s %s" % (phase, method))
        #             c_idx += 1
        #     plt.legend()
        #     plt.title("dataset%02d loss" % idx)
        #     pdf.savefig()
        #     plt.close()
