#! -*- coding: utf-8
import os.path as path
import os
import numpy as np

from glob import glob
from argparse import ArgumentParser
from concurrent.futures import ProcessPoolExecutor

if os.name == "posix":
    import matplotlib
    matplotlib.use("Agg")
    import matplotlib.pyplot as plt
else:
    import matplotlib.pyplot as plt


def __plot__(outdir, domain, queryfile, predictfile, format="png"):
    queries = np.load(queryfile)
    predicts = np.load(predictfile)
    if predicts.ndim > 2:
        predicts = predicts.squeeze(-(predicts.ndim-2))
    # predict結果の先頭にnanをパディング
    pad = np.tile(np.nan,
                  (len(predicts), queries.shape[1] - predicts.shape[1]))
    predicts = np.concatenate([pad, predicts], axis=-1)
    # グラフの描画範囲（y軸）を設定
    rate = 1.1
    tmp = np.vstack([queries, predicts])
    ylim = [-0.05, np.nanmax(tmp) * rate]

    for idx, (query, predict) in enumerate(zip(queries, predicts)):
        # query image
        imgfile = path.join(outdir, "%s.query.%03d.%s" % (domain, idx, format))
        plt.plot(query)
        plt.xlim(0, len(query))
        plt.axis("off")
        plt.savefig(imgfile)
        plt.close()

        # predict image
        imgfile = path.join(outdir, "%s.predict.%03d.%s" %
                            (domain, idx, format))
        plt.plot(predict)
        plt.xlim(0, len(predict))
        plt.axis("off")
        plt.savefig(imgfile)
        plt.close()

        # predict and target image
        imgfile = path.join(outdir, "%s.%03d.%s" % (domain, idx, format))
        plt.plot(query)
        plt.plot(predict)
        plt.xlim(0, len(query))
        plt.axis("off")
        plt.savefig(imgfile)
        plt.close()

    print(domain, "done")


if __name__ == "__main__":
    parser = ArgumentParser()
    parser.add_argument("rootdir", type=str)
    parser.add_argument("--targets",
                        default=["train"],
                        nargs="+", type=str,
                        choices=["train", "validation", "target"])
    parser.add_argument("--format", default="png", type=str,
                        choices=["png", "jpg", "svg", "eps"])
    args = parser.parse_args()
    print(args)

    executor = ProcessPoolExecutor()
    tasks = []

    for target in args.targets:
        for predictfile in glob(path.join(args.rootdir,
                                          target,
                                          "*.predict.npy")):
            parentdir = path.dirname(predictfile)
            domain, _ = path.splitext(path.basename(predictfile))
            domain = domain.replace(".predict", "")
            queryfile = path.join(
                parentdir, "%s.query.npy" % (domain))
            if not path.isfile(queryfile):
                queryfile = path.join(
                    parentdir, "%s.npy" % (domain))
                assert path.isfile(queryfile)

            outdir = path.join(parentdir, "img")
            if not path.isdir(outdir):
                os.makedirs(outdir)
            tasks.append(executor.submit(__plot__,
                                         outdir,
                                         domain,
                                         queryfile,
                                         predictfile, format=args.format))
            print("submit plot job %s %s" % (target, domain))

    for task in tasks:
        task.result()

    # parser = ArgumentParser()
    # parser.add_argument("rootdir", type=str)
    # parser.add_argument("--format", type=str,
    #                     default="png",
    #                     choices=["png", "eps", "svg"])
    # parser.add_argument("--DEBUG", default=False, action="store_true")
    # args = parser.parse_args()

    # tasks = []
    # executor = ProcessPoolExecutor()

    # datadirs = list(glob(path.join(args.rootdir, "dataset*", "*", "target")))
    # datadirs += list(glob(path.join(args.rootdir,
    #                                 "dataset*", "*", "*", "target")))
    # for datadir in datadirs:
    #     if not path.isdir(datadir):
    #         continue
    #     outdir = path.join(datadir, "img")
    #     if not path.isdir(outdir):
    #         os.makedirs(outdir)
    #     for query_file in glob(path.join(datadir, "*.query.npy")):
    #         predict_file = query_file.replace(".npy", "_predict.npy")
    #         assert path.isfile(
    #             query_file), "File %s does not found." % query_file
    #         assert path.isfile(
    #             predict_file), "File %s does not found." % predict_file

    #         task = executor.submit(
    #             __plot__, query_file, predict_file, outdir, format=args.format)
    #         tasks.append(task)

    #         if args.DEBUG:
    #             break
    #     if args.DEBUG:
    #         break

    # for task in tasks:
    #     task.result()
