#! -*- coding: utf-8
from concurrent.futures import ProcessPoolExecutor
from argparse import ArgumentParser
import numpy as np
import os
import os.path as path
from glob import iglob as glob

if os.name == "posix":
    import matplotlib
    matplotlib.use("Agg")
    from matplotlib import pyplot as plt
    from matplotlib.backends.backend_pdf import PdfPages
else:
    from matplotlib import pyplot as plt
    from matplotlib.backends.backend_pdf import PdfPages

def __plot__(outdir, domain, queryfile, predictfile, row=8):
    pdffile = path.join(outdir, "%s.pdf" % (domain))

    with PdfPages(pdffile) as pdf:
        fig, ax = plt.subplots(
            nrows=row, ncols=1, figsize=(16, 11), sharex=True)
        ax = np.array(ax).ravel()

        # target_data = np.load(target_file)
        # predict_data = np.load(predict_file)
        queries = np.load(queryfile)
        predicts = np.load(predictfile)
        if predicts.ndim > 2:
            predicts = predicts.squeeze(-(predicts.ndim-2))
        # predict結果の先頭にnanをパディング
        pad = np.tile(np.nan,
                      (len(predicts), queries.shape[1] - predicts.shape[1]))
        predicts = np.concatenate([pad, predicts], axis=-1)

        for i, (query, predict) in enumerate(zip(queries,
                                                 predicts)):
            idx = i % row
            ax[idx].plot(query, label="query")
            ax[idx].plot(predict, label="predict")

            ax[idx].axis("off")
            ax[idx].legend(fontsize=5)
            ax[idx].set_title("data no: %d" % i, fontsize=8)
            if idx == row-1:
                plt.suptitle("domain: %s" % domain)
                plt.tight_layout(rect=[0, 0.03, 1, 0.95])
                pdf.savefig()
                plt.close()
                fig, ax = plt.subplots(
                    nrows=row, ncols=1, figsize=(16, 11), sharex=True)
                ax = np.array(ax).ravel()
        for a in ax:
            a.axis("off")
        plt.suptitle("domain: %s" % domain)
        plt.tight_layout(rect=[0, 0.03, 1, 0.95])
        pdf.savefig()
        plt.close()

    print("plot %s done" % (domain))
    pass


if __name__ == "__main__":
    parser = ArgumentParser()
    parser.add_argument("rootdir", type=str)
    parser.add_argument("--targets",
                        default=["target"],
                        nargs="+", type=str,
                        choices=["train", "validation", "target"])
    args = parser.parse_args()
    print(args)

    executor = ProcessPoolExecutor()
    tasks = []

    for target in args.targets:
        for predictfile in glob(path.join(args.rootdir,
                                          target,
                                          "*.predict.npy")):
            parentdir = path.dirname(predictfile)
            domain, _ = path.splitext(path.basename(predictfile))
            domain = domain.replace(".predict", "")
            queryfile = path.join(
                parentdir, "%s.query.npy" % (domain))
            if not path.isfile(queryfile):
                queryfile = path.join(
                    parentdir, "%s.npy" % (domain))
                assert path.isfile(queryfile)

            tasks.append(executor.submit(__plot__,
                                         parentdir,
                                         domain,
                                         queryfile,
                                         predictfile))
            print("submit plot job %s %s" % (target, domain))

    for task in tasks:
        task.result()

