#! -*- coding: utf-8
import os
import os.path as path
import sys
from argparse import ArgumentParser
from datetime import datetime
from glob import glob
from logging import getLogger

import numpy as np
import pandas as pd

if os.name == "posix":
    import matplotlib
    matplotlib.use("Agg")
    from matplotlib import pyplot as plt
    from matplotlib.backends.backend_pdf import PdfPages
else:
    from matplotlib import pyplot as plt
    from matplotlib.backends.backend_pdf import PdfPages

if __name__ == "__main__":
    parser = ArgumentParser()
    parser.add_argument("pdf", type=str)
    parser.add_argument("score", type=str)
    parser.add_argument("--domain_specific", default=None, type=str)
    parser.add_argument("--cross_domain", default=None, type=str)
    parser.add_argument("--meta_learning", default=None, type=str)
    parser.add_argument("--diff_previous", default=None, type=str)
    parser.add_argument("--bidirectional", default=None, type=str)
    parser.add_argument("--attention", default=None, type=str)
    parser.add_argument("--attention_only", default=None, type=str)
    parser.add_argument("--linear", default=None, type=str)
    parser.add_argument("--nonlinear", default=None, type=str)
    parser.add_argument("--maml", default=None, type=str)
    parser.add_argument("--maml_linear", default=None, type=str)
    parser.add_argument("--maml_nonlinear", default=None, type=str)
    parser.add_argument("--domain_specific_linear", default=None, type=str)
    parser.add_argument("--domain_specific_nonlinear", default=None, type=str)
    parser.add_argument("--delimiter", type=str, default="\t")
    parser.add_argument("--num_freq", type=int, default=5)
    args = parser.parse_args()
    print(args)

    data_types, dirs = [], []
    for d, data_type in zip([args.domain_specific,
                             args.cross_domain,
                             args.meta_learning,
                             args.diff_previous,
                             args.bidirectional,
                             args.attention,
                             args.attention_only,
                             args.linear,
                             args.nonlinear,
                             args.maml,
                             args.maml_linear,
                             args.maml_nonlinear,
                             args.domain_specific_linear,
                             args.domain_specific_nonlinear],
                            ["domain_specific", "cross_domain",
                             "meta_learning", "diff_previous",
                             "bidirectional", "attention", "attention_only",
                             "linear", "nonlinear",
                             "maml", "maml_linear", "maml_nonlinear",
                             "domain_specific_linear",
                             "domain_specific_nonlinear"]):
        if d is not None:
            dataset_name = path.basename(path.dirname(d))
            dirs.append(d)
            data_types.append(data_type)
    npy_dir = None
    for d in [args.cross_domain, args.meta_learning, args.diff_previous, args.bidirectional, args.attention, args.linear, args.nonlinear, args.maml, args.maml_linear, args.maml_nonlinear]:
        if d is not None:
            npy_dir = d
            break
    assert npy_dir is not None and path.isdir(
        npy_dir), "%s not exists" % npy_dir

    # ドメイン別モデルの結果を連結
    if args.domain_specific is not None:
        score_file = path.join(args.domain_specific, "score.tsv")
        with open(score_file, "wt") as f:
            header = ["domain", "category", "type", "count", "error"]
            f.write(args.delimiter.join(header))
            f.write("\n")
            mean_score = {"target": [0.0, 0]}
            for score in glob(path.join(args.domain_specific, "*/score.tsv")):
                domain = path.basename(path.dirname(score))
                item = domain.split("_")
                if len(item) == 1:
                    domain, category = item[0], ""
                else:
                    domain, category = item[0], item[1]
                df = pd.read_csv(score, sep=args.delimiter)
                df.replace({"domain": {"\.(query|support)$": ""}},
                           inplace=True, regex=True)
                for train_type in ["target"]:
                    # col = "%s.query" % domain if train_type == "target" else domain
                    # row = df[(df["domain"] == col) & (
                    #     df["type"] == train_type)].iloc[0]
                    row = df[(df["domain"] == domain) & (
                        df["type"] == train_type)].iloc[0]
                    line = [row["domain"], row["category"],
                            row["type"], row["count"], row["error"]]
                    f.write(args.delimiter.join(map(str, line)))
                    f.write("\n")
                    mean_score[train_type][0] += row.error
                    mean_score[train_type][1] += 1

    # ドメイン別モデルの結果を連結
    if args.domain_specific_linear is not None:
        score_file = path.join(args.domain_specific_linear, "score.tsv")
        with open(score_file, "wt") as f:
            header = ["domain", "category", "type", "count", "error"]
            f.write(args.delimiter.join(header))
            f.write("\n")
            mean_score = {"target": [0.0, 0]}
            for score in glob(path.join(args.domain_specific_linear, "*/score.tsv")):
                domain = path.basename(path.dirname(score))
                item = domain.split("_")
                if len(item) == 1:
                    domain, category = item[0], ""
                else:
                    domain, category = item[0], item[1]
                df = pd.read_csv(score, sep=args.delimiter)
                df.replace({"domain": {"\.(query|support)$": ""}},
                           inplace=True, regex=True)
                for train_type in ["target"]:
                    # col = "%s.query" % domain if train_type == "target" else domain
                    # row = df[(df["domain"] == col) & (
                    #     df["type"] == train_type)].iloc[0]
                    row = df[(df["domain"] == domain) & (
                        df["type"] == train_type)].iloc[0]
                    line = [row["domain"], row["category"],
                            row["type"], row["count"], row["error"]]
                    f.write(args.delimiter.join(map(str, line)))
                    f.write("\n")
                    mean_score[train_type][0] += row.error
                    mean_score[train_type][1] += 1

    if args.domain_specific_nonlinear is not None:
        score_file = path.join(args.domain_specific_nonlinear, "score.tsv")
        with open(score_file, "wt") as f:
            header = ["domain", "category", "type", "count", "error"]
            f.write(args.delimiter.join(header))
            f.write("\n")
            mean_score = {"target": [0.0, 0]}
            for score in glob(path.join(args.domain_specific_nonlinear, "*/score.tsv")):
                domain = path.basename(path.dirname(score))
                item = domain.split("_")
                if len(item) == 1:
                    domain, category = item[0], ""
                else:
                    domain, category = item[0], item[1]
                df = pd.read_csv(score, sep=args.delimiter)
                df.replace({"domain": {"\.(query|support)$": ""}},
                           inplace=True, regex=True)
                for train_type in ["target"]:
                    # col = "%s.query" % domain if train_type == "target" else domain
                    # row = df[(df["domain"] == col) & (
                    #     df["type"] == train_type)].iloc[0]
                    row = df[(df["domain"] == domain) & (
                        df["type"] == train_type)].iloc[0]
                    line = [row["domain"], row["category"],
                            row["type"], row["count"], row["error"]]
                    f.write(args.delimiter.join(map(str, line)))
                    f.write("\n")
                    mean_score[train_type][0] += row.error
                    mean_score[train_type][1] += 1

    # 直前データとの比較
    if args.diff_previous is not None:
        if not path.isdir(args.diff_previous):
            os.makedirs(args.diff_previous)
        # meta learningディレクトリのデータを参照して、直前データと比較を行う
        with open(path.join(args.diff_previous, "score.tsv"), "wt") as f:
            header = ["domain", "category", "type", "count", "error"]
            f.write(args.delimiter.join(header))
            f.write("\n")
            for train_type in ["train", "validation", "target"]:
                for npy in glob(path.join(npy_dir, train_type, "*.npy")):
                    basename = path.basename(npy)
                    if ".support" in basename or "_predict" in basename or "_target" in basename or ".predict" in basename:
                        continue
                    domain, _ = path.splitext(path.basename(npy))
                    item = domain.split("_")
                    if len(item) == 1:
                        domain, category = item[0], ""
                    else:
                        domain, category = item[0], item[1]
                    domain = domain.replace(".query", "")
                    category = category.replace(".query", "")
                    # if ".query" in category:
                    #     domain = "%s.query" % domain
                    #     category = category.replace(".query", "")
                    # print(domain, category, npy)

                    datas = np.load(npy)
                    error = np.mean(
                        (datas[:, args.num_freq:-1] - datas[:, (args.num_freq + 1):]) ** 2)
                    count = np.prod(datas[:, args.num_freq:-1].shape)
                    f.write(args.delimiter.join(
                        map(str, [domain, category, train_type, count, error])))
                    f.write("\n")

    df = None
    for data_type, score_dir in zip(data_types, dirs):
        if score_dir is None:
            print("skip %s %s" % (data_type, score_dir))
            continue
        scoref = path.join(score_dir, "score.tsv")
        print(scoref)
        tmp = pd.read_csv(scoref,
                          sep=args.delimiter,
                          dtype={"domain": str, "category": str, "type": str})
        tmp = tmp.loc[tmp["domain"] != tmp["type"],
                      ["domain", "category", "type", "error"]]
        tmp.rename(columns={"error": data_type}, inplace=True)
        if df is None:
            df = tmp
        else:
            df = pd.merge(
                df, tmp, on=["domain", "category", "type"], how="outer")

    pdfdir = path.dirname(args.pdf)
    df.to_csv(path.join(pdfdir, "score.csv"), index=False, sep=",")
    width_unit = 0.4
    with PdfPages(args.pdf) as pdf:
        for train_type in ["train", "validation", "target"]:
            datas = df[df["type"] == train_type].sort_values(
                ["domain", "category"])
            # datas.sort_values(["domain", "category"], inplace=True)
            fig, ax = plt.subplots(figsize=(width_unit * len(datas), 9))
            for data_type in data_types:
                ax.plot(
                    range(len(datas)), datas.loc[:, data_type], label=data_type.replace("_", " "))
            xlabels = ["%s %s" % (row.domain, row.category)
                       for i, row in datas.iterrows()]
            plt.xticks(ticks=range(len(datas)), labels=xlabels,
                       rotation=30, ha="right", fontsize=5)
            plt.title("error of %s domains" % train_type)
            plt.tight_layout()
            plt.legend()
            pdf.savefig()
            plt.close()
            datas.to_csv(path.join(pdfdir, "%s.csv" % train_type), index=False)

    with open(args.score, "wt") as f:
        header = ["dataset"]
        header.extend(data_types)
        f.write(args.delimiter.join(header))
        f.write("\n")
        line = [dataset_name]
        datas = df[df["type"] == "target"]
        line.extend([np.nanmean(datas.loc[:, data_type].values)
                     for data_type in data_types])
        f.write(args.delimiter.join(map(str, line)))
        f.write("\n")
