#! -*- coding: utf-8
import os
import os.path as path
import sys
from argparse import ArgumentParser
from datetime import datetime
from glob import glob
from logging import getLogger

import numpy as np
import pandas as pd

if os.name == "posix":
    import matplotlib
    matplotlib.use("Agg")
    from matplotlib import pyplot as plt
    from matplotlib.backends.backend_pdf import PdfPages
else:
    from matplotlib import pyplot as plt
    from matplotlib.backends.backend_pdf import PdfPages

if __name__ == "__main__":
    parser = ArgumentParser()
    # parser.add_argument("meta_learning", type=str)
    # parser.add_argument("bidirectional", type=str)
    parser.add_argument("attention", type=str)
    parser.add_argument("maml", type=str)
    parser.add_argument("maml_linear", type=str)
    parser.add_argument("maml_nonlinear", type=str)
    parser.add_argument("pdf", type=str)
    parser.add_argument("score", type=str)
    parser.add_argument("--delimiter", type=str, default="\t")
    parser.add_argument("--num_freq", type=int, default=5)
    args = parser.parse_args()

    # dataset_name = path.basename(path.dirname(args.meta_learning))
    dataset_name = path.basename(path.dirname(args.attention))

    # data_types = ["meta_learning", "bidirectional", "attention"]
    # dirs = [args.meta_learning, args.bidirectional, args.attention]
    data_types = ["attention", "maml", "maml_linear", "maml_nonlinear"]
    dirs = [args.attention, args.maml, args.maml_linear, args.maml_nonlinear]
    df = None
    for data_type, score_dir in zip(data_types, dirs):
        tmp = pd.read_csv(path.join(score_dir, "score.tsv"),
                          sep=args.delimiter,
                          dtype={"domain": str, "category": str, "type": str})
        tmp = tmp.loc[tmp["domain"] != tmp["type"],
                      ["domain", "category", "type", "error"]]
        tmp.rename(columns={"error": data_type}, inplace=True)
        if df is None:
            df = tmp
        else:
            df = pd.merge(
                df, tmp, on=["domain", "category", "type"], how="outer")

    pdfdir = path.dirname(args.pdf)
    df.to_csv(path.join(pdfdir, "score.csv"), index=False, sep=",")
    width_unit = 0.4
    with PdfPages(args.pdf) as pdf:
        for train_type in ["train", "validation", "target"]:
            datas = df[df["type"] == train_type].sort_values(
                ["domain", "category"])
            # datas.sort_values(["domain", "category"], inplace=True)
            fig, ax = plt.subplots(figsize=(width_unit * len(datas), 9))
            for data_type in data_types:
                ax.plot(
                    range(len(datas)), datas.loc[:, data_type], label=data_type.replace("_", " "))
            xlabels = ["%s %s" % (row.domain, row.category)
                       for i, row in datas.iterrows()]
            plt.xticks(ticks=range(len(datas)), labels=xlabels,
                       rotation=30, ha="right", fontsize=5)
            plt.title("error of %s domains" % train_type)
            plt.tight_layout()
            plt.legend()
            pdf.savefig()
            plt.close()
            datas.to_csv(path.join(pdfdir, "%s.csv" % train_type), index=False)

    with open(args.score, "wt") as f:
        header = ["dataset"]
        header.extend(data_types)
        f.write(args.delimiter.join(header))
        f.write("\n")
        line = [dataset_name]
        datas = df[df["type"] == "target"]
        line.extend([np.nanmean(datas.loc[:, data_type].values)
                     for data_type in data_types])
        f.write(args.delimiter.join(map(str, line)))
        f.write("\n")
