# -*- coding: utf-8
import logging
import os
import sys
from argparse import ArgumentParser
from datetime import datetime
from glob import glob
from logging import getLogger
from os import path

import numpy as np

domains = ["ACSF1", "Adiac",
           "AllGestureWiimoteX", "AllGestureWiimoteY", "AllGestureWiimoteZ",
           "ArrowHead", "BME", "Beef", "BeetleFly", "BirdChicken", "CBF", "Car",
           "Chinatown", "ChlorineConcentration", "CinCECGTorso", "Coffee",
           "Computers", "CricketX", "CricketY", "CricketZ", "Crop",
           "DiatomSizeReduction", "DistalPhalanxOutlineAgeGroup", "DistalPhalanxOutlineCorrect",
           "DistalPhalanxTW", "DodgerLoopDay", "DodgerLoopGame", "DodgerLoopWeekend",
           "ECG200", "ECG5000", "ECGFiveDays", "EOGHorizontalSignal", "EOGVerticalSignal",
           "Earthquakes", "ElectricDevices", "EthanolLevel", "FaceAll", "FaceFour", "FacesUCR",
           "FiftyWords", "Fish", "FordA", "FordB", "FreezerRegularTrain", "FreezerSmallTrain",
           "Fungi", "GestureMidAirD1", "GestureMidAirD2", "GestureMidAirD3", "GesturePebbleZ1",
           "GesturePebbleZ2", "GunPoint", "GunPointAgeSpan", "GunPointMaleVersusFemale",
           "GunPointOldVersusYoung", "Ham", "HandOutlines", "Haptics", "Herring", "HouseTwenty",
           "InlineSkate", "InsectEPGRegularTrain", "InsectEPGSmallTrain", "InsectWingbeatSound",
           "ItalyPowerDemand", "LargeKitchenAppliances", "Lightning2", "Lightning7",
           "Mallat", "Meat", "MedicalImages", "MelbournePedestrian", "MiddlePhalanxOutlineAgeGroup",
           "MiddlePhalanxOutlineCorrect", "MiddlePhalanxTW", "MixedShapesRegularTrain",
           "MixedShapesSmallTrain", "MoteStrain", "NonInvasiveFetalECGThorax1",
           "NonInvasiveFetalECGThorax2", "OSULeaf", "OliveOil", "PLAID",
           "PhalangesOutlinesCorrect", "Phoneme", "PickupGestureWiimoteZ", "PigAirwayPressure",
           "PigArtPressure", "PigCVP", "Plane", "PowerCons", "ProximalPhalanxOutlineAgeGroup",
           "ProximalPhalanxOutlineCorrect", "ProximalPhalanxTW", "RefrigerationDevices",
           "Rock", "ScreenType", "SemgHandGenderCh2", "SemgHandMovementCh2", "SemgHandSubjectCh2",
           "ShakeGestureWiimoteZ", "ShapeletSim", "ShapesAll", "SmallKitchenAppliances",
           "SmoothSubspace", "SonyAIBORobotSurface1", "SonyAIBORobotSurface2", "StarLightCurves",
           "Strawberry", "SwedishLeaf", "Symbols", "SyntheticControl",
           "ToeSegmentation1", "ToeSegmentation2", "Trace", "TwoLeadECG", "TwoPatterns", "UMD",
           "UWaveGestureLibraryAll", "UWaveGestureLibraryX", "UWaveGestureLibraryY", "UWaveGestureLibraryZ",
           "Wafer", "Wine", "WordSynonyms", "Worms", "WormsTwoClass", "Yoga"]
loglevels = ["TRACE0", "TRACE1", "TRACE2", "TRACE3", "TRACE4",
             "TRACE5", "TRACE6", "TRACE7", "TRACE8", "TRACE9",
             "DEBUG", "INFO", "WARNING", "ERROR", "CRITICAL"]
# logformat = "[%(asctime)s] <%(levelname)8s> %(process)d %(module)s.%(funcName)s %(message)s"
logformat = "[%(asctime)s] <%(levelname)s> %(message)s"

##### Global logger settings #####
for i in range(0, 10):  # 独自追加したログレベルの名称をloggingパッケージに登録
    name = loglevels[i]
    logging.addLevelName(i, name)
    setattr(logging, name, i)  # ログ名を設定
##################################

if __name__ == "__main__":
    ###### 共通オプション ######
    common_args = ArgumentParser(add_help=False)
    common_args.add_argument("--predict_freq", type=int,
                             default=1, help="何時間先のデータを予測するか")
    common_args.add_argument("--delimiter", type=str,
                             default=",", help="データファイルのデリミタ")
    common_args.add_argument("--num_query", type=int,
                             default=None, help="使用する時系列数の上限")
    common_args.add_argument("--num_train_support",
                             type=int, default=None, help="")
    common_args.add_argument("--num_predict_support",
                             type=int, default=None, help="")
    common_args.add_argument("--num_ar_feature", type=int, default=3, help="")
    common_args.add_argument(
        "--query_seq_len", type=int, default=None, help="")
    common_args.add_argument(
        "--support_seq_len", type=int, default=None, help="")
    common_args.add_argument("--seed", type=int, default=None, help="乱数種")
    common_args.add_argument("--RETRY", default=False,
                             action="store_true", help="再学習を行う")
    common_args.add_argument("--DEBUG", default=False,
                             action="store_true", help="デバッグモード")
    common_args.add_argument("--logfile", type=str, default=None,
                             help="ログ出力先。未指定の場合、標準エラーにログを出力")
    common_args.add_argument("--loglevel", type=str,
                             default="INFO", choices=loglevels, help="ログレベル")
    ############ モデルパラメータ ############
    model_param_args = ArgumentParser(add_help=False)
    model_param_args.add_argument("--hidden_size", type=int,
                                  default=32, help="LSTMの隠れ層")
    model_param_args.add_argument("--num_layers", type=int,
                                  default=1, help="LSTMの反復レイヤ数")
    ############ ドメイン特徴量抽出モデル用のモデルパラメータ ############
    domain_model_param_args = ArgumentParser(add_help=False)
    domain_model_param_args.add_argument("--domain_hidden_size", type=int,
                                         default=32, help="ドメイン特徴量抽出モデル用のLSTMの隠れ層")
    domain_model_param_args.add_argument("--domain_num_layers", type=int,
                                         default=1, help="ドメイン特徴量抽出モデル用のLSTMの反復レイヤ数")
    domain_model_param_args.add_argument("--domain_output_size", type=int,
                                         default=16, help="ドメイン特徴量抽出モデルの出力次元数")
    domain_model_param_args.add_argument("--domain_forward_unit00",
                                         type=int, default=64, help="")
    domain_model_param_args.add_argument("--domain_forward_unit01",
                                         type=int, default=32, help="")
    domain_model_param_args.add_argument("--domain_forward_unit02",
                                         type=int, default=16, help="")
    ############ 学習パラメータ ############
    train_param_args = ArgumentParser(add_help=False)
    train_param_args.add_argument("--batch_size", type=int,
                                  default=50, help="バッチサイズ")
    train_param_args.add_argument("--CUDA", default=False, action="store_true",
                                  help="CUDAを使用して学習を行う。未指定の場合、CPUを使用する。指定時かつCUDAが利用できない場合、エラーとする。")
    train_param_args.add_argument("--workers",
                                  type=int, default=1, help="ワーカスレッド数")
    ###### ドメインモード オプション ######
    domain_specific_common_args = ArgumentParser(add_help=False)
    domain_specific_common_args.add_argument(
        "root_dir", type=str, help="入力データディレクトリ")
    domain_specific_common_args.add_argument(
        "domain", type=str, help="対象ドメイン名", metavar="domain")
    domain_specific_common_args.add_argument(
        "model", type=str, help="")
    domain_specific_common_args.add_argument(
        "outdir", type=str, help="結果の出力ディレクトリ")

    cross_domain_common_args = ArgumentParser(add_help=False)
    cross_domain_common_args.add_argument(
        "root_dir", type=str, help="入力データディレクトリ")
    cross_domain_common_args.add_argument(
        "model", type=str, help="")
    cross_domain_common_args.add_argument(
        "outdir", type=str, help="結果の出力ディレクトリ")
    cross_domain_common_args.add_argument(
        "--train_domain", type=int, default=None, help="学習データに使用するドメイン数")

    parser = ArgumentParser()
    domain_modes = parser.add_subparsers(dest="domain_mode")

    domain_specific_subcommands = ["specific",
                                   "specific_linear",
                                   "specific_nonlinear"]
    for subcommand in domain_specific_subcommands:
        domain_specific_args = domain_modes.add_parser(subcommand)
        data_modes = domain_specific_args.add_subparsers(dest="data_mode")
        fewshot_args = data_modes.add_parser("fewshot",
                                             parents=[common_args,
                                                      domain_specific_common_args,
                                                      model_param_args,
                                                      train_param_args],
                                             help="Few-shot learning用に、trainデータのみ件数指定を行う。validation, testデータは残りのデータから割合で分割する")
        longterm_args = data_modes.add_parser("longterm",
                                              parents=[common_args,
                                                       domain_specific_common_args,
                                                       model_param_args,
                                                       train_param_args],
                                              help="長時間予測モデル")
        longterm_args.add_argument(
            "--input_seq", type=int, default=10, help="")

    cross_domain_subcommands = ["cross", "linear", "nonlinear"]
    for subcommand in cross_domain_subcommands:
        subcommand_args = domain_modes.add_parser(subcommand)
        data_modes = subcommand_args.add_subparsers(dest="data_mode")
        fewshot_args = data_modes.add_parser("fewshot",
                                             parents=[common_args,
                                                      cross_domain_common_args,
                                                      model_param_args,
                                                      train_param_args],
                                             help="Few-shot learning用に、trainデータのみ件数指定を行う。validation, testデータは残りのデータから割合で分割する")
        longterm_args = data_modes.add_parser("longterm",
                                              parents=[common_args,
                                                       cross_domain_common_args,
                                                       model_param_args,
                                                       train_param_args],
                                              help="長時間予測モデル")
        longterm_args.add_argument(
            "--input_seq", type=int, default=10, help="")

    meta_subcommands = ["meta", "bidirectional", "attention"]
    for subcommand in meta_subcommands:
        meta_learn_args = domain_modes.add_parser(subcommand)
        data_modes = meta_learn_args.add_subparsers(dest="data_mode")
        fewshot_args = data_modes.add_parser("fewshot",
                                             parents=[common_args,
                                                      cross_domain_common_args,
                                                      model_param_args,
                                                      domain_model_param_args,
                                                      train_param_args],
                                             help="Few-shot learning用に、trainデータのみ件数指定を行う。validation, testデータは残りのデータから割合で分割する")
        longterm_args = data_modes.add_parser("longterm",
                                              parents=[common_args,
                                                       cross_domain_common_args,
                                                       model_param_args,
                                                       domain_model_param_args,
                                                       train_param_args],
                                              help="長時間予測モデル")
        longterm_args.add_argument(
            "--input_seq", type=int, default=10, help="")

    maml_subcommands = ["maml", "maml_linear", "maml_nonlinear"]
    for subcommand in maml_subcommands:
        maml_args = domain_modes.add_parser(subcommand)
        data_modes = maml_args.add_subparsers(dest="data_mode")
        fewshot_args = data_modes.add_parser("fewshot",
                                             parents=[common_args,
                                                      cross_domain_common_args,
                                                      model_param_args,
                                                      train_param_args],
                                             help="Few-shot learning用に、trainデータのみ件数指定を行う。validation, testデータは残りのデータから割合で分割する")
        fewshot_args.add_argument("--inner_iter", type=int,
                                  default=5, help="")
        fewshot_args.add_argument("--inner_learning_rate", type=float,
                                  default=0.005, help="内部イテレーション用の学習率")
        longterm_args = data_modes.add_parser("longterm",
                                              parents=[common_args,
                                                       cross_domain_common_args,
                                                       model_param_args,
                                                       train_param_args],
                                              help="長時間予測モデル")
        longterm_args.add_argument("--inner_iter", type=int,
                                   default=5, help="")
        longterm_args.add_argument("--inner_learning_rate", type=float,
                                   default=0.005, help="内部イテレーション用の学習率")
        longterm_args.add_argument(
            "--input_seq", type=int, default=10, help="")

    args = parser.parse_args()

    if not "domain_mode" in args or args.domain_mode is None:
        raise ValueError("Domain mode not set")
    if not "data_mode" in args or args.data_mode is None:
        raise ValueError("Data mode not set")

    # set loglevel and log output.
    loglevel = getattr(logging, args.loglevel, None) if hasattr(
        args, "loglevel") else "DEBUG"
    logfile = args.logfile if hasattr(args, "logfile") else None
    assert not loglevel is None  # choicesを指定しているので、このassertには引っかからない
    if logfile is None:
        logging.basicConfig(level=loglevel, format=logformat,
                            stream=sys.stderr)  # ログファイル未指定の場合、標準エラーに出力
    else:
        logdir = path.dirname(logfile)
        if logdir is not None and len(logdir) > 0 and not path.isdir(logdir):
            os.makedirs(logdir)
        logging.basicConfig(level=loglevel, format=logformat,
                            filename=logfile)  # ログファイル未指定の場合、標準エラーに出力

    logger = logging.getLogger(__name__)
    del logging  # 呼び出し間違い禁止
    logger.info(args)

    # 各種パラメータの確認
    assert args.predict_freq >= 0  # 入力時系列最後とデータを予測することも可とする
    assert args.batch_size > 0

    if args.outdir is not None and len(args.outdir) > 0 and not path.isdir(args.outdir):
        os.makedirs(args.outdir)

    scorefile = path.join(args.outdir, "score.tsv")
    checkpoint = args.model

    if args.domain_mode in ["specific", "specific_linear", "specific_nonlinear"]:
        if args.data_mode in ["fewshot", "longterm"]:
            if args.data_mode == "fewshot":
                if args.domain_mode == "specific":
                    from trainers import LstmTrainer as Trainer
                elif args.domain_mode == "specific_linear":
                    from trainers import LinearTrainer as Trainer
                elif args.domain_mode == "specific_nonlinear":
                    from trainers import NonLinearTrainer as Trainer
                else:
                    raise ValueError("Unknown sub command %s" %
                                     (args.domain_mode))
            elif args.data_mode == "longterm":
                if args.domain_mode == "specific":
                    from trainers.longterm import LongTermLstmTrainer as Trainer
                elif args.domain_mode == "specific_linear":
                    from trainers.longterm import LongTermLinearTrainer as Trainer
                elif args.domain_mode == "specific_nonlinear":
                    from trainers.longterm import LongTermNonLinearTrainer as Trainer
                else:
                    raise ValueError("Unknown sub command %s" %
                                     (args.domain_mode))
        else:
            raise ValueError("Unknown mode %s" % (args.data_mode))

        trainer = Trainer(
            predict_freq=args.predict_freq,
            num_query=args.num_query,
            num_train_support=0,
            num_predict_support=0,
            support_seq_len=args.support_seq_len,
            query_seq_len=args.query_seq_len,
            num_ar_feature=args.num_ar_feature,
            batch_size=args.batch_size,
            workers=args.workers,
            use_cuda=args.CUDA)

        trainer.create_datas_specific(args.root_dir,
                                      args.outdir,
                                      args.domain, None,
                                      seed=args.seed, num_support=args.num_train_support)

    elif args.domain_mode in ["cross", "linear", "nonlinear"]:
        if args.data_mode in ["fewshot", "longterm"]:
            if args.data_mode == "fewshot":
                if args.domain_mode == "cross":
                    from trainers import LstmTrainer as Trainer
                elif args.domain_mode == "linear":
                    from trainers import LinearTrainer as Trainer
                elif args.domain_mode == "nonlinear":
                    from trainers import NonLinearTrainer as Trainer
                else:
                    raise ValueError("Unknown sub command %s" %
                                     (args.domain_mode))
            if args.data_mode == "longterm":
                if args.domain_mode == "cross":
                    from trainers.longterm import LongTermLstmTrainer as Trainer
                elif args.domain_mode == "linear":
                    from trainers.longterm import LongTermLinearTrainer as Trainer
                elif args.domain_mode == "nonlinear":
                    from trainers.longterm import LongTermNonLinearTrainer as Trainer
                else:
                    raise ValueError("Unknown sub command %s" %
                                     (args.domain_mode))
        else:
            raise ValueError("Unknown mode %s" % (args.data_mode))

        trainer = Trainer(
            predict_freq=args.predict_freq,
            num_query=args.num_query,
            num_train_support=0,
            num_predict_support=0,
            support_seq_len=args.support_seq_len,
            query_seq_len=args.query_seq_len,
            num_ar_feature=args.num_ar_feature,
            batch_size=args.batch_size,
            workers=args.workers,
            use_cuda=args.CUDA)
        trainer.create_datas(args.root_dir, args.outdir,
                             train_domains=args.train_domain,
                             seed=args.seed)

    elif args.domain_mode in ["meta", "bidirectional", "attention"]:
        if args.data_mode in ["fewshot", "longterm"]:
            if args.data_mode == "fewshot":
                if args.domain_mode == "meta":
                    from trainers import MetaLearnTrainer as Trainer
                elif args.domain_mode == "bidirectional":
                    from trainers import BidirectionalTrainer as Trainer
                elif args.domain_mode == "attention":
                    from trainers import AttentionTrainer as Trainer
                else:
                    raise ValueError("Unknown sub command %s" %
                                     (args.domain_mode))
            if args.data_mode == "longterm":
                if args.domain_mode == "meta":
                    from trainers.longterm import LongTermMetaLearnTrainer as Trainer
                elif args.domain_mode == "bidirectional":
                    from trainers.longterm import LongTermBidirectionalTrainer as Trainer
                elif args.domain_mode == "attention":
                    from trainers.longterm import LongTermAttentionTrainer as Trainer
                else:
                    raise ValueError("Unknown sub command %s" %
                                     (args.domain_mode))
        else:
            raise ValueError("Unknown mode %s" % (args.data_mode))

        trainer = Trainer(
            predict_freq=args.predict_freq,
            num_query=args.num_query,
            num_train_support=args.num_train_support,
            num_predict_support=args.num_predict_support,
            support_seq_len=args.support_seq_len,
            query_seq_len=args.query_seq_len,
            num_ar_feature=args.num_ar_feature,
            batch_size=args.batch_size,
            workers=args.workers,
            use_cuda=args.CUDA)
        trainer.create_datas(args.root_dir, args.outdir,
                             train_domains=args.train_domain,
                             seed=args.seed)

    elif args.domain_mode in ["maml", "maml_linear", "maml_nonlinear"]:
        if args.data_mode in ["fewshot", "longterm"]:
            if args.data_mode == "fewshot":
                if args.domain_mode == "maml":
                    from trainers import MamlTrainer as Trainer
                elif args.domain_mode == "maml_linear":
                    from trainers import MamlLinearTrainer as Trainer
                elif args.domain_mode == "maml_nonlinear":
                    from trainers import MamlNonLinearTrainer as Trainer
                else:
                    raise ValueError("Unknown sub command %s" %
                                     (args.domain_mode))
            if args.data_mode == "longterm":
                if args.domain_mode == "maml":
                    from trainers.longterm import LongTermMamlTrainer as Trainer
                elif args.domain_mode == "maml_linear":
                    from trainers.longterm import LongTermMamlLinearTrainer as Trainer
                elif args.domain_mode == "maml_nonlinear":
                    from trainers.longterm import LongTermMamlNonLinearTrainer as Trainer
                else:
                    raise ValueError("Unknown sub command %s" %
                                     (args.domain_mode))
        else:
            raise ValueError("Unknown mode %s" % (args.data_mode))

        trainer = Trainer(
            predict_freq=args.predict_freq,
            num_query=args.num_query,
            num_train_support=args.num_train_support,
            num_predict_support=args.num_predict_support,
            support_seq_len=args.support_seq_len,
            query_seq_len=args.query_seq_len,
            num_ar_feature=args.num_ar_feature,
            batch_size=args.batch_size,
            workers=args.workers,
            use_cuda=args.CUDA)
        trainer.create_datas(args.root_dir, args.outdir,
                             train_domains=args.train_domain,
                             seed=args.seed)
    else:
        raise ValueError("Unknown domain mode %s" % args.domain_mode)

    if "input_seq" in args:
        trainer.input_sequence = args.input_seq
    if not path.isfile(scorefile) or path.getsize(scorefile) == 0 or args.RETRY:
        argdict = vars(args)
        del argdict["model"]
        trainer.predict(checkpoint,
                        input_size=1,
                        reduction="sum", **argdict)
