#! -*- coding: utf-8
""" 単純に１つ前のデータと次のデータとの誤差を計算するスクリプト
"""
import sys
import os
import os.path as path
from argparse import ArgumentParser
import numpy as np
import logging

if os.name == "posix":
    import matplotlib
    matplotlib.use("Agg")
    from matplotlib import pyplot as plt
    from matplotlib.backends.backend_pdf import PdfPages
else:
    from matplotlib import pyplot as plt
    from matplotlib.backends.backend_pdf import PdfPages

domains = ["ACSF1", "Adiac", "AllGestureWiimoteX", "AllGestureWiimoteY", "AllGestureWiimoteZ", "ArrowHead", "BME", "Beef", "BeetleFly", "BirdChicken", "CBF", "Car", "Chinatown", "ChlorineConcentration", "CinCECGTorso", "Coffee", "Computers", "CricketX", "CricketY", "CricketZ", "Crop", "DiatomSizeReduction", "DistalPhalanxOutlineAgeGroup", "DistalPhalanxOutlineCorrect", "DistalPhalanxTW", "DodgerLoopDay", "DodgerLoopGame", "DodgerLoopWeekend", "ECG200", "ECG5000", "ECGFiveDays", "EOGHorizontalSignal", "EOGVerticalSignal", "Earthquakes", "ElectricDevices", "EthanolLevel", "FaceAll", "FaceFour", "FacesUCR", "FiftyWords", "Fish", "FordA", "FordB", "FreezerRegularTrain", "FreezerSmallTrain", "Fungi", "GestureMidAirD1", "GestureMidAirD2", "GestureMidAirD3", "GesturePebbleZ1", "GesturePebbleZ2", "GunPoint", "GunPointAgeSpan", "GunPointMaleVersusFemale", "GunPointOldVersusYoung", "Ham", "HandOutlines", "Haptics", "Herring", "HouseTwenty", "InlineSkate", "InsectEPGRegularTrain", "InsectEPGSmallTrain", "InsectWingbeatSound", "ItalyPowerDemand", "LargeKitchenAppliances", "Lightning2", "Lightning7",
           "Mallat", "Meat", "MedicalImages", "MelbournePedestrian", "MiddlePhalanxOutlineAgeGroup", "MiddlePhalanxOutlineCorrect", "MiddlePhalanxTW", "MixedShapesRegularTrain", "MixedShapesSmallTrain", "MoteStrain", "NonInvasiveFetalECGThorax1", "NonInvasiveFetalECGThorax2", "OSULeaf", "OliveOil", "PLAID", "PhalangesOutlinesCorrect", "Phoneme", "PickupGestureWiimoteZ", "PigAirwayPressure", "PigArtPressure", "PigCVP", "Plane", "PowerCons", "ProximalPhalanxOutlineAgeGroup", "ProximalPhalanxOutlineCorrect", "ProximalPhalanxTW", "RefrigerationDevices", "Rock", "ScreenType", "SemgHandGenderCh2", "SemgHandMovementCh2", "SemgHandSubjectCh2", "ShakeGestureWiimoteZ", "ShapeletSim", "ShapesAll", "SmallKitchenAppliances", "SmoothSubspace", "SonyAIBORobotSurface1", "SonyAIBORobotSurface2", "StarLightCurves", "Strawberry", "SwedishLeaf", "Symbols", "SyntheticControl", "ToeSegmentation1", "ToeSegmentation2", "Trace", "TwoLeadECG", "TwoPatterns", "UMD", "UWaveGestureLibraryAll", "UWaveGestureLibraryX", "UWaveGestureLibraryY", "UWaveGestureLibraryZ", "Wafer", "Wine", "WordSynonyms", "Worms", "WormsTwoClass", "Yoga"]
loglevels = ["TRACE0", "TRACE1", "TRACE2", "TRACE3", "TRACE4", "TRACE5", "TRACE6", "TRACE7", "TRACE8", "TRACE9",
             "DEBUG", "INFO", "WARNING", "ERROR", "CRITICAL"]
logformat = "[%(asctime)s] <%(levelname)8s> %(process)d %(module)s.%(funcName)s %(message)s"

##### Global logger settings #####
for i in range(0, 10):  # 独自追加したログレベルの名称をloggingパッケージに登録
    name = loglevels[i]
    logging.addLevelName(i, name)
    setattr(logging, name, i)  # ログ名を設定
##################################


def preb1(domain, root_dir, outdir, delimiter="\t"):
    datafile = path.join(root_dir, "%s_normalized.tsv" % (domain))
    pdffile = path.join(outdir, "%s.pdf" % (domain))
    scorefile = path.join(outdir, "score.tsv")
    row = 8
    header = ["domain", "type", "count", "error"]
    with open(datafile) as inf, PdfPages(pdffile) as pdf, open(scorefile, "wt") as outf:
        fig, ax = plt.subplots(
            nrows=row, ncols=1, figsize=(16, 11), sharex=True)
        ax = np.array(ax).ravel()
        lines = inf.readlines()
        error, count = 0.0, 0
        for i, datas in enumerate(lines):
            datas = np.array(list(map(float, datas.split(delimiter))))  # float変換
            # １つ前のデータをそのまま予測値として、正解データとの誤差を算出する
            predicts, targets = datas[:-1], datas[1:]
            error += np.sum((predicts - targets) ** 2)  # 誤差集計
            count += len(predicts)  # データ数のカウント

            # ===== プロット =====
            idx = i % row
            ax[idx].plot(targets, label="target")
            ax[idx].plot(predicts, label="predict")
            ax[idx].axis("off")
            ax[idx].legend(fontsize=5)
            ax[idx].set_title("data no: %d" % i, fontsize=8)
            if idx == row-1:
                plt.suptitle("domain: %s" % domain)
                plt.tight_layout(rect=[0, 0.03, 1, 0.95])
                pdf.savefig()
                plt.close()
                fig, ax = plt.subplots(
                    nrows=row, ncols=1, figsize=(16, 11), sharex=True)
                ax = np.array(ax).ravel()
        for a in ax:
            a.axis("off")
        plt.suptitle("domain: %s" % domain)
        plt.tight_layout(rect=[0, 0.03, 1, 0.95])
        pdf.savefig()
        plt.close()
        # ===== プロット =====

        # ===== 誤差出力 =====
        error /= count
        outf.write(delimiter.join(header))
        outf.write("\n")
        outf.write(delimiter.join(map(str, [domain, "preb1", count, error])))
        outf.write("\n")


if __name__ == "__main__":
    common_args = ArgumentParser(add_help=False)  # 各サブコマンド共通の引数
    common_args.add_argument("root_dir", type=str, help="入力データディレクトリ")
    common_args.add_argument(
        "domain", type=str, choices=domains, help="対象ドメイン名")
    common_args.add_argument("outdir", type=str, help="結果の出力ディレクトリ")
    common_args.add_argument("--delimiter", type=str,
                             default="\t", help="データファイルのデリミタ")
    ###### ログ設定用引数 ######
    common_args.add_argument("--logfile", type=str,
                             default=None, help="ログ出力先。未指定の場合、標準エラーにログを出力")
    common_args.add_argument("--loglevel", type=str,
                             default="DEBUG", choices=loglevels, help="ログレベル")

    parser = ArgumentParser()
    subcommands = parser.add_subparsers(dest="command")
    preb1_parser = subcommands.add_parser(
        "preb1", parents=[common_args], help="単純に１つ前のデータと比較する")

    args = parser.parse_args()

    # set loglevel and log output.
    loglevel = getattr(logging, args.loglevel, None)
    assert not loglevel is None  # choicesを指定しているので、このassertには引っかからない
    if args.logfile is None:
        logging.basicConfig(level=loglevel, format=logformat,
                            stream=sys.stderr)  # ログファイル未指定の場合、標準エラーに出力
    else:
        logdir = path.dirname(args.logfile)
        if logdir is not None and len(logdir) > 0 and not path.isdir(logdir):
            os.makedirs(logdir)
        logging.basicConfig(level=loglevel, format=logformat,
                            filename=args.logfile)  # ログファイル未指定の場合、標準エラーに出力

    logger = logging.getLogger(__name__)
    del logging  # 呼び出し間違い禁止
    logger.info(args)

    if not path.isdir(args.outdir):
        os.makedirs(args.outdir)

    if args.command == "preb1":
        preb1(args.domain, args.root_dir, args.outdir, delimiter=args.delimiter)
    else:
        raise ValueError("Unknown command %s" % args.command)
