#! -*- coding: utf-8
import os
import os.path as path
from argparse import ArgumentParser
import numpy as np
from glob import iglob as glob
from tempfile import TemporaryFile
from concurrent.futures import ProcessPoolExecutor


def __check_cols__(tsvfile, delimiter="\t"):
    datas = np.loadtxt(tsvfile, delimiter=delimiter)
    return datas.shape[1] - 1  # 先頭にカテゴリが含まれているので、カテゴリ分の長さを減算


# def __proc__(tsvfile, outdir, max_col, support=5, delimiter="\t", seed=None):
#     # 元ファイルから1カラム目（カテゴリ）を削除し、
#     basename, ext = path.splitext(path.basename(tsvfile))
#     supprotfile = path.join(outdir, "%s%s" % (basename, ".support"))
#     queryfile = path.join(outdir, "%s%s" % (basename, ".query"))

#     datas = np.loadtxt(tsvfile, delimiter=delimiter)
#     datas = datas[:, 1:]  # １カラム目（カテゴリ）削除
#     padding_size = max_col - datas.shape[1]

#     mean = np.nanmean(datas)
#     var = np.nanvar(datas)
#     datas = (datas - mean) / var
#     if padding_size > 0:
#         datas = np.concatenate(
#             [datas, np.tile(np.nan, (len(datas), padding_size))], axis=1)

#     indexs = np.arange(len(datas), dtype=int)
#     np.random.seed(seed)
#     support_indexs = np.random.choice(indexs, support, replace=False)
#     indexs = np.setdiff1d(indexs, support_indexs)
#     query_indexs = indexs
#     del indexs

#     np.save(supprotfile, datas[support_indexs])
#     np.save(queryfile, datas[query_indexs])

def __proc__(tsvfile, outdir, max_col, train=0.6, val=0.1, test=0.3, support=5, max_datas=100, delimiter="\t", seed=None):
    # 元ファイルから1カラム目（カテゴリ）を削除し、
    basename, ext = path.splitext(path.basename(tsvfile))
    trainfile = path.join(outdir, "%s%s" % (basename, ".train"))
    validationfile = path.join(outdir, "%s%s" % (basename, ".validation"))
    supprotfile = path.join(outdir, "%s%s" % (basename, ".support"))
    queryfile = path.join(outdir, "%s%s" % (basename, ".query"))

    datas = np.loadtxt(tsvfile, delimiter=delimiter)
    datas = datas[:, 1:]  # １カラム目（カテゴリ）削除
    padding_size = max_col - datas.shape[1]

    mean = np.nanmean(datas)
    var = np.nanvar(datas)
    datas = (datas - mean) / var
    if padding_size > 0:
        datas = np.concatenate(
            [datas, np.tile(np.nan, (len(datas), padding_size))], axis=1)
    # 全体のデータ数からtrain, validation, support, queryデータに分割する
    # supportは個数指定
    # 全体データからsupportを引いた残りから割合でtrain, validation, queryに分割する
    # train, validationのデータ数が指定データ数（max_datas）より多い場合、queryに残データを押し付ける
    data_length = len(datas) - support  # 全体データからsupportを引いた残り
    assert data_length > 3  # 3分割できない場合、エラー
    __sum__ = train + val + test
    num_train = int(data_length * (train / __sum__))
    num_val = int(data_length * (val / __sum__))
    # train, validation, testデータの件数を算出。それぞれ最低1件必要
    if num_train < 1:
        num_train = 1
    elif num_train > max_datas:  # train, validationデータ数に上限を設定
        num_train = max_datas
    if num_val < 1:
        num_val = 1
    elif num_val > max_datas:  # train, validationデータ数に上限を設定
        num_val = max_datas
    del __sum__
    num_test = data_length - num_train
    assert num_train > 0 and num_val > 0 and num_test

    indexs_query = np.arange(len(datas))
    indexs_support = np.random.choice(indexs_query, support, replace=False)
    indexs_query = np.setdiff1d(indexs_query, indexs_support)  # 選択されたindexを除外
    indexs_train = np.random.choice(indexs_query, support, replace=False)
    indexs_query = np.setdiff1d(indexs_query, indexs_train)  # 選択されたindexを除外
    indexs_val = np.random.choice(indexs_query, support, replace=False)
    indexs_query = np.setdiff1d(indexs_query, indexs_val)  # 選択されたindexを除外
    assert len(datas) == (len(indexs_train) + len(indexs_val) +
                          len(indexs_support) + len(indexs_query))

    np.save(trainfile, datas[indexs_train])
    np.save(validationfile, datas[indexs_val])
    np.save(supprotfile, datas[indexs_support])
    np.save(queryfile, datas[indexs_query])


if __name__ == "__main__":
    """ データをsupport/queryに分割する。
    supportは評価には使用しない。
    """
    parser = ArgumentParser()
    parser.add_argument("rootdir", type=str)
    parser.add_argument("--outdir", type=str, default=None)
    parser.add_argument("--train", type=float, default=0.6)
    parser.add_argument("--validation", type=float, default=0.1)
    parser.add_argument("--test", type=float, default=0.3)
    parser.add_argument("--max_datas", type=int, default=100)
    parser.add_argument("--support", type=int, default=5)
    parser.add_argument("--seed", type=int, default=None)
    args = parser.parse_args()
    # print(args)
    if args.outdir is None:
        args.outdir = args.rootdir
    if not path.isdir(args.outdir):
        os.makedirs(args.outdir)

    tasks = []
    executor = ProcessPoolExecutor()

    max_col = []
    for filepath in glob(path.join(args.rootdir, "*.tsv")):
        if "_normalized" in filepath:
            continue  # 処理済みファイルは除外
        tasks.append(executor.submit(__check_cols__, filepath))
    for task in tasks:
        max_col.append(task.result())
    max_col = np.max(max_col)
    print("max sequence length: %d" % max_col)

    for filepath in glob(path.join(args.rootdir, "*.tsv")):
        if "_normalized" in filepath:
            continue  # 処理済みファイルは除外
        tasks.append(executor.submit(__proc__, filepath, args.outdir, max_col,
                                    train=args.train, val=args.validation, test=args.test, support=args.support, max_datas=args.max_datas, seed=args.seed))

        # def __proc__(tsvfile, outdir, max_col, support=5, delimiter="\t", seed=None):
        # tasks.append(executor.submit(__proc__, filepath, args.outdir, max_col,
        #                              support=args.support, seed=args.seed))
    for task in tasks:
        task.result()
