#! -*- coding: utf-8
import os
import os.path as path
from argparse import ArgumentParser
from glob import glob

import numpy as np
import pandas as pd

domain_methods = ["attention", "cross_domain", "maml",
                  "linear", "nonlinear",
                  "maml_linear", "maml_nonlinear",
                  "domain_specific", "domain_specific_linear", "domain_specific_nonlinear",
                  "attention_only", "bidirectional", "meta_learning",
                  "diff_previous"]
support_methods = ["attention", "maml", "maml_linear", "maml_nonlinear",
                   "diff_previous"]
if __name__ == "__main__":
    common_args = ArgumentParser(add_help=False)
    common_args.add_argument("rootdir", type=str)
    common_args.add_argument("outdir", type=str)
    common_args.add_argument("--ndataset", type=int, default=30)
    parser = ArgumentParser()
    subparser = parser.add_subparsers(dest="command")
    domain_args = subparser.add_parser("domain", parents=[common_args])
    domain_args.add_argument("--domains", type=int, nargs="+",
                             default=[10, 20, 30, 40, 50, 55])
    domain_args.add_argument("--methods", type=str, nargs="+",
                             default=domain_methods,
                             choices=domain_methods)
    support_args = subparser.add_parser("support", parents=[common_args])
    support_args.add_argument("--supports", type=int, nargs="+",
                              default=[1, 2, 3, 4, 5, 6, 7, 8, 9, 10])
    support_args.add_argument("--methods", type=str, nargs="+",
                              default=support_methods,
                              choices=domain_methods)
    args = parser.parse_args()

    if not path.isdir(args.outdir):
        os.makedirs(args.outdir, exist_ok=True)

    scorefile = path.join(args.outdir, "%s_per_domain_scores" % args.command)
    methodlist = path.join(args.outdir, "%s_methods.txt" % args.command)
    domainlist = path.join(args.outdir, "domainlist.txt")
    with open(methodlist, "wt") as f:
        f.write(",".join(args.methods))

    values = []
    cols = ["dataset", "domain"] + args.methods
    if args.command == "domain":
        dirs = []
        # for ndomain in args.domains:
        #     dirs.append(path.join(
        #         args.rootdir, "train%d_predict_support5_data30_seq100" % ndomain))
        dirs.append(path.join(
            args.rootdir, "train55_predict_support5_data30_seq100"))
    elif args.command == "support":
        dirs = []
        for nsupport in args.supports:
            dirs.append(path.join(
                args.rootdir, "train55_predict_support%d_data30_seq100" % nsupport))

    domains, ndomains = None, None
    for target_dir in dirs:
        print(target_dir)
        scores = pd.read_csv(path.join(target_dir, "score_domain_dataset.csv"))
        scores = scores.rename(columns={"cross_doamin": "cross_domain"})
        # print(scores["cross_domain"])
        # print(scores.columns)
        scores = scores.loc[:, cols]
        # print(scores)
        domainset = set(scores["domain"].values)
        if domains is None:
            domains = sorted(list(domainset))
            ndomains = len(domains)
        assert set(domains) == domainset

        domain_values = []
        for d in range(args.ndataset):
            dataset = "dataset%02d" % d
            score = scores[scores["dataset"] ==
                           dataset].copy().reset_index(drop=True)
            score.drop(columns=["dataset"], inplace=True)
            # print(score)
            # targetに含まれていないdomainを算出
            untarget_domains = np.setdiff1d(domains, score["domain"].values)
            # print(untarget_domains)
            for domain in untarget_domains:
                score.loc[len(score)] = [domain] + [np.nan] * \
                    (len(score.columns) - 1)
            score.sort_values(by="domain", inplace=True)
            v = score.drop(columns=["domain"]).values  # domain x method
            domain_values.append(v.T)  # -> method x domain
        v = np.stack(domain_values, axis=-1)
        # print(v.shape)
        values.append(v)
    values = np.stack(values, axis=0)
    print(values)
    print(values.shape)
    np.save(scorefile, values)
    if not path.isfile(domainlist):
        with open(domainlist, "wt") as f:
            f.write(",".join(domains))
            f.write("\n")

    # for di, domain in enumerate(domains):
    #     for si in range(values.shape[0]):
    #         for mi, method in enumerate(args.methods):
    #             s = values[si, mi, di].ravel()
    #             c = len(s[~np.isnan(s)])
    #             s = np.nanmean(s)
    #             print("ax%d %s %s %f %d" % (si+1, domain, method, s, c))
    #         # print("ax%d %s %s" %
    #         #       (si+1, domain, np.sum(values[si, :, di], axis=1)))
    for di, domain in enumerate(domains):
        for si in range(values.shape[0]):
            print("ax%d %s %s" %
                  (si+1, domain, values[si, :, di]))

    # items = path.basename(args.rootdir).split("_")
    # nsupport = int(items[2].replace("support", ""))
    # basename = "%s_%s" % (items[0], items[2])

    # cols = ["dataset", "domain"]
    # cols.extend(args.methods)
    # scores = pd.read_csv(path.join(args.rootdir, "score_domain_dataset.csv"),
    #                      usecols=cols)
    # # print(scores)
    # domains = sorted(list(set(scores["domain"].values)))
    # print("domains: %s" % (", ".join(domains)))
    # domainfile = path.join(args.outdir, "domains.txt")
    # if not path.isfile(domainfile):
    #     with open(domainfile, "wt") as f:
    #         f.write(",".join(domains))
    #         f.write("\n")

    # values = []
    # for d in range(args.ndataset):
    #     dataset = "dataset%02d" % d
    #     score = scores[scores["dataset"] ==
    #                    dataset].copy().reset_index(drop=True)
    #     score.drop(columns=["dataset"], inplace=True)
    #     # print(score)
    #     # targetに含まれていないdomainを算出
    #     untarget_domains = np.setdiff1d(domains, score["domain"].values)
    #     # print(untarget_domains)
    #     for domain in untarget_domains:
    #         score.loc[len(score)] = [domain] + [np.nan] * \
    #             (len(score.columns) - 1)
    #     score.sort_values(by="domain", inplace=True)
    #     # print(score)
    #     v = score.drop(columns=["domain"]).values
    #     # print(dataset, v.shape)
    #     values.append(v)
    #     if d > 2:
    #         break

    # values = np.stack(values, axis=-1)
    # # print(values)
    # # print(values.shape)
    # filename = path.join(args.outdir, basename)
    # np.save(filename, values)
    # # print(np.nanmean(values, axis=2))
    # # print(np.nanvar(values, axis=2))
