#! -*- coding: utf-8
import os
import os.path as path
from argparse import ArgumentParser
from glob import glob

import numpy as np
import pandas as pd

domain_methods = ["attention", "cross_domain", "maml",
                  "linear", "nonlinear",
                  "maml_linear", "maml_nonlinear",
                  "domain_specific", "domain_specific_linear", "domain_specific_nonlinear",
                  "diff_previous"]
support_methods = ["attention", "maml", "maml_linear", "maml_nonlinear",
                   "diff_previous"]
if __name__ == "__main__":
    common_args = ArgumentParser(add_help=False)
    common_args.add_argument("rootdir", type=str)
    common_args.add_argument("outdir", type=str)
    common_args.add_argument("--ndataset", type=int, default=30)
    parser = ArgumentParser()
    subparser = parser.add_subparsers(dest="command")
    domain_args = subparser.add_parser("domain", parents=[common_args])
    domain_args.add_argument("--domains", type=int, nargs="+",
                             default=[10, 20, 30, 40, 50, 55])
    domain_args.add_argument("--methods", type=str, nargs="+",
                             default=domain_methods,
                             choices=domain_methods)
    support_args = subparser.add_parser("support", parents=[common_args])
    support_args.add_argument("--supports", type=int, nargs="+",
                              default=[1, 2, 3, 4, 5, 6, 7, 8, 9, 10])
    support_args.add_argument("--methods", type=str, nargs="+",
                              default=support_methods,
                              choices=domain_methods)
    args = parser.parse_args()

    if not path.isdir(args.outdir):
        os.makedirs(args.outdir, exist_ok=True)

    scorefile = path.join(args.outdir, "%s_socres" % args.command)
    methodlist = path.join(args.outdir, "%s_methods.txt" % args.command)
    with open(methodlist, "wt") as f:
        f.write(",".join(args.methods))

    values = []
    cols = ["dataset"] + args.methods
    if args.command == "domain":
        dirs = []
        for ndomain in args.domains:
            dirs.append(path.join(
                args.rootdir, "train%d_predict_support5_data30_seq100" % ndomain))
    elif args.command == "support":
        dirs = []
        for nsupport in args.supports:
            dirs.append(path.join(
                args.rootdir, "train55_predict_support%d_data30_seq100" % nsupport))

    for target_dir in dirs:
        score = pd.read_csv(path.join(target_dir, "score.csv"),
                            usecols=cols, nrows=args.ndataset).loc[:, cols]
        score.sort_values(by=["dataset"])
        # print(score)
        values.append(score.loc[:, args.methods].values.T)
    values = np.stack(values, axis=0)
    print(values.shape)
    print(values[-1, 0, :])
    np.save(scorefile, values)
