#! -*- coding: utf-8
import os
import os.path as path
from argparse import ArgumentParser
from concurrent.futures import ProcessPoolExecutor
from glob import glob

import numpy as np
import pandas as pd

__methods__ = ["attention", "maml", "cross_domain"]
__tags__ = ["train", "validation", "target"]


def __proc_time__(rootdir, outdir, method, sep="\t"):
    dfs = []
    outfile = path.join(outdir, "train_proc_%s.log" % method)
    for file in glob(path.join(rootdir, "dataset*", method, "train_proc.log")):
        logs = None
        dataset = path.basename(path.dirname(path.dirname(file)))
        df = pd.read_csv(file, sep=sep)
        df.drop(columns=["batch", "loss"], inplace=True)
        for tag in __tags__:
            log = df.loc[df["tag"] == tag].copy()
            # 必要なカラム以外削除
            log.drop(columns=["tag"], inplace=True)
            log.rename(columns={"proc_time": tag}, inplace=True)
            log = log.groupby(by="epoch").sum().reset_index(drop=False)
            if logs is None:
                logs = log
            else:
                logs = pd.merge(logs, log, on="epoch")
            del log
        logs["dataset"] = dataset
        logs["proc_time"] = logs.loc[:, __tags__].sum(axis=1)
        logs = logs.loc[:, ["epoch", "dataset", "proc_time"] + __tags__]
        dfs.append(logs)
    df = pd.concat(dfs)
    df.drop(columns=["epoch"], inplace=True)
    df = df.groupby(by=["dataset"]).sum().reset_index(drop=False)
    df.to_csv(outfile, index=False)
    return outfile, method
    # procs = df["proc_time"].values
    # mean, var = np.mean(procs), np.var(procs)
    # return method, mean, var


if __name__ == "__main__":
    parser = ArgumentParser()
    parser.add_argument("rootdir", type=str,)
    parser.add_argument("outdir", type=str,)
    parser.add_argument("--methods", type=str,
                        nargs="+",
                        default=["attention", "maml", "cross_domain"],
                        choices=__methods__)
    parser.add_argument("--sep", type=str, default="\t")
    args = parser.parse_args()

    os.makedirs(path.dirname(args.outdir), exist_ok=True)
    # os.makedirs(path.dirname(args.outfile), exist_ok=True)

    executor = ProcessPoolExecutor()
    tasks = [executor.submit(__proc_time__,
                             args.rootdir, args.outdir, method, sep=args.sep)
             for method in args.methods]
    logs = []
    for task in tasks:
        proc_log_file, method = task.result()
        print(proc_log_file, method)
        df = pd.read_csv(proc_log_file)
        df.drop(columns=["dataset"], inplace=True)
        means = df.mean(axis=0)
        means["method"] = method
        logs.append(means)
    logs = pd.DataFrame(logs)
    logs = logs.loc[:, ["method", "proc_time",
                        "train", "validation", "target"]]
    logs.to_csv(path.join(args.outdir, "train_proc.log"), index=False)

    # with open(args.outfile, "wt") as f:
    #     f.write(",".join(["dataset", "method", "proc_time",
    #                       "train", "validation", "test"]))
    #     f.write("\n")
    #     for task in tasks:
    #         ret = task.result()
    #         print(ret)
    #         f.write(",".join(map(str, ret)))
    #         f.write("\n")
