# -*- coding: utf-8
import logging
import os
import sys
from argparse import ArgumentParser
from datetime import datetime
from logging import getLogger
from os import path

import numpy as np

domains = ["ACSF1", "Adiac", "AllGestureWiimoteX", "AllGestureWiimoteY", "AllGestureWiimoteZ", "ArrowHead", "BME", "Beef", "BeetleFly", "BirdChicken", "CBF", "Car", "Chinatown", "ChlorineConcentration", "CinCECGTorso", "Coffee", "Computers", "CricketX", "CricketY", "CricketZ", "Crop", "DiatomSizeReduction", "DistalPhalanxOutlineAgeGroup", "DistalPhalanxOutlineCorrect", "DistalPhalanxTW", "DodgerLoopDay", "DodgerLoopGame", "DodgerLoopWeekend", "ECG200", "ECG5000", "ECGFiveDays", "EOGHorizontalSignal", "EOGVerticalSignal", "Earthquakes", "ElectricDevices", "EthanolLevel", "FaceAll", "FaceFour", "FacesUCR", "FiftyWords", "Fish", "FordA", "FordB", "FreezerRegularTrain", "FreezerSmallTrain", "Fungi", "GestureMidAirD1", "GestureMidAirD2", "GestureMidAirD3", "GesturePebbleZ1", "GesturePebbleZ2", "GunPoint", "GunPointAgeSpan", "GunPointMaleVersusFemale", "GunPointOldVersusYoung", "Ham", "HandOutlines", "Haptics", "Herring", "HouseTwenty", "InlineSkate", "InsectEPGRegularTrain", "InsectEPGSmallTrain", "InsectWingbeatSound", "ItalyPowerDemand", "LargeKitchenAppliances", "Lightning2", "Lightning7",
           "Mallat", "Meat", "MedicalImages", "MelbournePedestrian", "MiddlePhalanxOutlineAgeGroup", "MiddlePhalanxOutlineCorrect", "MiddlePhalanxTW", "MixedShapesRegularTrain", "MixedShapesSmallTrain", "MoteStrain", "NonInvasiveFetalECGThorax1", "NonInvasiveFetalECGThorax2", "OSULeaf", "OliveOil", "PLAID", "PhalangesOutlinesCorrect", "Phoneme", "PickupGestureWiimoteZ", "PigAirwayPressure", "PigArtPressure", "PigCVP", "Plane", "PowerCons", "ProximalPhalanxOutlineAgeGroup", "ProximalPhalanxOutlineCorrect", "ProximalPhalanxTW", "RefrigerationDevices", "Rock", "ScreenType", "SemgHandGenderCh2", "SemgHandMovementCh2", "SemgHandSubjectCh2", "ShakeGestureWiimoteZ", "ShapeletSim", "ShapesAll", "SmallKitchenAppliances", "SmoothSubspace", "SonyAIBORobotSurface1", "SonyAIBORobotSurface2", "StarLightCurves", "Strawberry", "SwedishLeaf", "Symbols", "SyntheticControl", "ToeSegmentation1", "ToeSegmentation2", "Trace", "TwoLeadECG", "TwoPatterns", "UMD", "UWaveGestureLibraryAll", "UWaveGestureLibraryX", "UWaveGestureLibraryY", "UWaveGestureLibraryZ", "Wafer", "Wine", "WordSynonyms", "Worms", "WormsTwoClass", "Yoga"]
loglevels = ["TRACE0", "TRACE1", "TRACE2", "TRACE3", "TRACE4", "TRACE5", "TRACE6", "TRACE7", "TRACE8", "TRACE9",
             "DEBUG", "INFO", "WARNING", "ERROR", "CRITICAL"]
logformat = "[%(asctime)s] <%(levelname)8s> %(process)d %(module)s.%(funcName)s %(message)s"

##### Global logger settings #####
for i in range(0, 10):  # 独自追加したログレベルの名称をloggingパッケージに登録
    name = loglevels[i]
    logging.addLevelName(i, name)
    setattr(logging, name, i)  # ログ名を設定
##################################

if __name__ == "__main__":
    ###### 共通オプション ######
    common_args = ArgumentParser(add_help=False)
    common_args.add_argument("--predict_freq", type=int,
                             default=1, help="何時間先のデータを予測するか")
    common_args.add_argument("--delimiter", type=str,
                             default=",", help="データファイルのデリミタ")
    common_args.add_argument("--max_datas", type=int,
                             default=None, help="使用する時系列数の上限")
    common_args.add_argument("--seed", type=int, default=None, help="乱数種")
    common_args.add_argument("--RETRY", default=False,
                             action="store_true", help="再学習を行う")
    common_args.add_argument("--DEBUG", default=False,
                             action="store_true", help="デバッグモード")
    common_args.add_argument("--logfile", type=str, default=None,
                             help="ログ出力先。未指定の場合、標準エラーにログを出力")
    common_args.add_argument("--loglevel", type=str,
                             default="INFO", choices=loglevels, help="ログレベル")
    ############ モデルパラメータ ############
    model_param_args = ArgumentParser(add_help=False)
    model_param_args.add_argument("--hidden_size", type=int,
                                  default=32, help="LSTMの隠れ層")
    model_param_args.add_argument("--num_layers", type=int,
                                  default=1, help="LSTMの反復レイヤ数")
    ############ ドメイン特徴量抽出モデル用のモデルパラメータ ############
    domain_model_param_args = ArgumentParser(add_help=False)
    domain_model_param_args.add_argument("--domain_hidden_size", type=int,
                                         default=32, help="ドメイン特徴量抽出モデル用のLSTMの隠れ層")
    domain_model_param_args.add_argument("--domain_num_layers", type=int,
                                         default=1, help="ドメイン特徴量抽出モデル用のLSTMの反復レイヤ数")
    domain_model_param_args.add_argument("--domain_output_size", type=int,
                                         default=16, help="ドメイン特徴量抽出モデルの出力次元数")
    ############ 学習パラメータ ############
    train_param_args = ArgumentParser(add_help=False)
    train_param_args.add_argument("--batch_size", type=int,
                                  default=32, help="バッチサイズ")
    train_param_args.add_argument(
        "--epochs", type=int, default=500, help="学習回数")
    train_param_args.add_argument("--min_delta", type=float, default=0.0,
                                  help="学習打ち切り時の、loss更新閾値。このパラメータで指定された値よりlossが改善されている場合、lossが更新されたものとみなす")
    train_param_args.add_argument("--patience", type=int, default=5000,
                                  help="最低学習回数。このパラメータで指定された学習回数中に最良のvalidation lossが更新されない場合、学習を打ち切る。")
    train_param_args.add_argument("--learning_rate", type=float,
                                  default=0.001, help="学習率")
    train_param_args.add_argument("--CUDA", default=False, action="store_true",
                                  help="CUDAを使用して学習を行う。未指定の場合、CPUを使用する。指定時かつCUDAが利用できない場合、エラーとする。")
    train_param_args.add_argument(
        "--workers", type=int, default=4, help="ワーカスレッド数")
    train_param_args.add_argument("--restore_best_weight", default=False, action="store_true",
                                  help="学習終了時に最もvalidation lossがよかったepochのweight、biasを復元する")

    ###### ドメインモード オプション ######
    domain_specific_common_args = ArgumentParser(add_help=False)
    domain_specific_common_args.add_argument(
        "root_dir", type=str, help="入力データディレクトリ")
    domain_specific_common_args.add_argument("domain", type=str,
                                             choices=domains, help="対象ドメイン名", metavar="domain")
    domain_specific_common_args.add_argument(
        "outdir", type=str, help="結果の出力ディレクトリ")

    cross_domain_common_args = ArgumentParser(add_help=False)
    cross_domain_common_args.add_argument(
        "root_dir", type=str, help="入力データディレクトリ")
    cross_domain_common_args.add_argument(
        "outdir", type=str, help="結果の出力ディレクトリ")
    cross_domain_common_args.add_argument(
        "--train_domain", type=int, default=100, help="学習データに使用するドメイン数")
    cross_domain_common_args.add_argument(
        "--validation_domain", type=int, default=10, help="検証データに使用するドメイン数")

    parser = ArgumentParser()
    domain_modes = parser.add_subparsers(dest="domain_mode")
    domain_specific_args = domain_modes.add_parser("specific")
    data_modes = domain_specific_args.add_subparsers(dest="data_mode")
    full_args = data_modes.add_parser("full", parents=[common_args, domain_specific_common_args, model_param_args, train_param_args],
                                      help="指定された割合でtrain, validation, testようにデータを分割する")
    full_args.add_argument("--data_percentage", type=float, nargs=3, default=(75., 10., 15.),
                           help="train, validation, testデータの割合。データ各時系列データの先頭からtrain, validation, testの順に指定された値の割合で分割する")
    full_args.add_argument("--data_seed", type=int,
                           default=None, help="データ分割時の乱数の種")
    fewshot_args = data_modes.add_parser("fewshot", parents=[common_args, domain_specific_common_args, model_param_args, train_param_args],
                                         help="Few-shot learning用に、trainデータのみ件数指定を行う。validation, testデータは残りのデータから割合で分割する")

    cross_domain_args = domain_modes.add_parser("cross")
    data_modes = cross_domain_args.add_subparsers(dest="data_mode")
    full_args = data_modes.add_parser("full", parents=[common_args, cross_domain_common_args, model_param_args, train_param_args],
                                      help="指定された割合でtrain, validation, testようにデータを分割する")
    full_args.add_argument("--domain_seed", type=int,
                           default=None, help="データ分割時の乱数の種")
    fewshot_args = data_modes.add_parser("fewshot", parents=[common_args, cross_domain_common_args, model_param_args, train_param_args],
                                         help="Few-shot learning用に、trainデータのみ件数指定を行う。validation, testデータは残りのデータから割合で分割する")
    fewshot_args.add_argument(
        "--domain_seed", type=int, default=None, help="データ分割時の乱数の種")

    meta_learn_args = domain_modes.add_parser("meta")
    data_modes = meta_learn_args.add_subparsers(dest="data_mode")
    fewshot_args = data_modes.add_parser("fewshot", parents=[common_args, cross_domain_common_args, model_param_args, domain_model_param_args, train_param_args],
                                         help="Few-shot learning用に、trainデータのみ件数指定を行う。validation, testデータは残りのデータから割合で分割する")
    fewshot_args.add_argument(
        "--domain_seed", type=int, default=None, help="データ分割時の乱数の種")

    args = parser.parse_args()

    if not "domain_mode" in args or args.domain_mode is None:
        raise ValueError("Domain mode not set")
    if not "data_mode" in args or args.data_mode is None:
        raise ValueError("Data mode not set")

    # set loglevel and log output.
    loglevel = getattr(logging, args.loglevel, None) if hasattr(
        args, "loglevel") else "DEBUG"
    logfile = args.logfile if hasattr(args, "logfile") else None
    assert not loglevel is None  # choicesを指定しているので、このassertには引っかからない
    if logfile is None:
        logging.basicConfig(level=loglevel, format=logformat,
                            stream=sys.stderr)  # ログファイル未指定の場合、標準エラーに出力
    else:
        logdir = path.dirname(logfile)
        if logdir is not None and len(logdir) > 0 and not path.isdir(logdir):
            os.makedirs(logdir)
        logging.basicConfig(level=loglevel, format=logformat,
                            filename=logfile)  # ログファイル未指定の場合、標準エラーに出力

    logger = logging.getLogger(__name__)
    del logging  # 呼び出し間違い禁止
    logger.info(args)

    # 各種パラメータの確認
    assert args.predict_freq >= 0  # 入力時系列最後とデータを予測することも可とする
    assert args.learning_rate > 0.000
    assert args.batch_size > 0 and args.epochs > 0 and args.patience > 0

    if args.outdir is not None and len(args.outdir) > 0 and not path.isdir(args.outdir):
        os.makedirs(args.outdir)

    modelfile = path.join(args.outdir, "model.pt")
    scorefile = path.join(args.outdir, "score.tsv")

    if args.domain_mode == "specific":

        if args.data_mode == "full":
            from trainers import UCRArchiveTrainer as Trainer
            assert np.sum(
                [p > 0.0 for p in args.data_percentage]) == len(args.data_percentage)
            train_per, val_per, test_per = args.data_percentage
            trainer = Trainer()
            datafile = path.join(args.root_dir, "%s_%s.npy" %
                                 (args.domain, "normalized"))
            trainer.split_data(datafile, args.outdir,
                               train_per, val_per, test_per, seed=args.data_seed)
            if args.RETRY or not path.isfile(scorefile):
                trainer.train(modelfile, epochs=args.epochs, batch_size=args.batch_size, min_delta=args.min_delta, max_datas=args.max_datas,
                              patience=args.patience, restore_best_weight=args.restore_best_weight,
                              workers=args.workers, predict_freq=args.predict_freq, use_cuda=args.CUDA, seed=args.seed,
                              input_size=1, hidden_size=args.hidden_size, num_layers=args.num_layers, lr=args.learning_rate, reduction="sum")
                trainer.predict(args.outdir, modelfile, batch_size=args.batch_size, workers=args.workers,
                                predict_freq=args.predict_freq, use_cuda=args.CUDA, input_size=1, hidden_size=args.hidden_size, num_layers=args.num_layers)
        elif args.data_mode == "fewshot":
            # assert args.train > 0 and args.validation > 0.0 and args.test > 0.0
            from trainers import UCRArchiveCrossDomainTrainer as Trainer
            trainer = Trainer()

            # 出力ディレクトリに対してtrain, validation, testディレクトリを作成
            # 各ファイルへのシンボリックリンクを作成
            for data_type, filelist in zip(["train", "validation", "query"], [trainer.trains, trainer.validations, trainer.tests]):
                basename = "%s.%s.npy" % (args.domain, data_type)
                target_org_file = path.join(args.root_dir, basename)
                assert path.isfile(
                    target_org_file), "filr %s not found" % target_org_file
                target_dir = path.join(args.outdir, data_type)
                target_file = path.join(target_dir, basename)
                if not path.isdir(target_dir):
                    os.makedirs(target_dir)
                if not path.isfile(target_file) and not path.islink(target_file):
                    os.symlink(path.abspath(target_org_file), target_file)
                filelist.append(target_file)
            if not path.isfile(modelfile) or args.RETRY:
                trainer.train(modelfile, epochs=args.epochs, batch_size=args.batch_size, min_delta=args.min_delta, max_datas=args.max_datas,
                              patience=args.patience, restore_best_weight=args.restore_best_weight,
                              workers=args.workers, predict_freq=args.predict_freq, use_cuda=args.CUDA, seed=args.seed,
                              input_size=1, hidden_size=args.hidden_size, num_layers=args.num_layers, lr=args.learning_rate, reduction="sum")
            if not path.isfile(scorefile) or args.RETRY:
                trainer.predict_targets = ([trainer.trains, trainer.validations, trainer.tests], [
                                           "train", "validation", "query"])
                trainer.predict(args.outdir, modelfile, batch_size=args.batch_size, workers=args.workers, ext="npy",
                                predict_freq=args.predict_freq, use_cuda=args.CUDA, input_size=1, hidden_size=args.hidden_size, num_layers=args.num_layers)
            # trainer.plot(args.outdir, replot=True, row=8)
        else:
            raise ValueError("Unknown data mode %s" % args.data_mode)
    elif args.domain_mode == "cross":
        assert len(domains) - args.train_domain - args.validation_domain > 0
        modelfile = path.join(args.outdir, "model.pt")
        if args.data_mode == "full":
            from trainers import UCRArchiveCrossDomainTrainer as Trainer
            trainer = Trainer()
            trainer.split_data(args.root_dir, train_domain=args.train_domain,
                               validation_domain=args.validation_domain, seed=args.domain_seed)
            if args.DEBUG:
                if len(trainer.trains) > 3:
                    del trainer.trains[3:]
                if len(trainer.validations) > 3:
                    del trainer.validations[3:]
                if len(trainer.tests) > 3:
                    del trainer.tests[3:]
            print(trainer.trains)
            print(trainer.validations)
            print(trainer.tests)
            if not args.RETRY and path.isfile(scorefile):
                trainer.train(modelfile, epochs=args.epochs, batch_size=args.batch_size, min_delta=args.min_delta, max_datas=args.max_datas, patience=args.patience,
                              restore_best_weight=args.restore_best_weight, workers=args.workers, predict_freq=args.predict_freq,
                              use_cuda=args.CUDA, seed=args.seed, input_size=1, hidden_size=args.hidden_size, num_layers=args.num_layers, lr=args.learning_rate, reduction="sum")
                trainer.predict(args.outdir, modelfile, batch_size=args.batch_size, workers=args.workers,
                                predict_freq=args.predict_freq, use_cuda=args.CUDA, input_size=1, hidden_size=args.hidden_size, num_layers=args.num_layers)
        elif args.data_mode == "fewshot":
            from trainers import UCRArchiveCrossDomainTrainer as Trainer
            trainer = Trainer()

            np.random.seed(args.domain_seed)
            train_domains = np.random.choice(
                domains, args.train_domain, replace=False)
            test_domains = np.setdiff1d(domains, train_domains)
            val_domains = np.random.choice(
                test_domains, args.validation_domain, replace=False)
            test_domains = np.setdiff1d(test_domains, val_domains)
            for data_type, target_domains, filelist in zip(["train", "validation", "query"], [train_domains, val_domains, test_domains], [trainer.trains, trainer.validations, trainer.tests]):
                for domain in target_domains:
                    basename = "%s.%s.npy" % (domain, data_type)
                    target_org_file = path.join(args.root_dir, basename)
                    assert path.isfile(target_org_file)
                    target_dir = path.join(args.outdir, data_type)
                    target_file = path.join(target_dir, basename)
                    if not path.isdir(target_dir):
                        os.makedirs(target_dir)
                    if not path.isfile(target_file) and not path.islink(target_file):
                        os.symlink(path.abspath(target_org_file), target_file)
                    filelist.append(target_file)

            if not path.isfile(modelfile) or args.RETRY:
                trainer.train(modelfile, epochs=args.epochs, batch_size=args.batch_size, min_delta=args.min_delta, max_datas=args.max_datas, patience=args.patience,
                              restore_best_weight=args.restore_best_weight,
                              workers=args.workers, predict_freq=args.predict_freq, use_cuda=args.CUDA, seed=args.seed,
                              input_size=1, hidden_size=args.hidden_size, num_layers=args.num_layers, lr=args.learning_rate, reduction="sum")
            if not path.isfile(scorefile) or args.RETRY:
                trainer.predict_targets = ([trainer.trains, trainer.validations, trainer.tests], [
                                           "train", "validation", "query"])
                trainer.predict(args.outdir, modelfile, batch_size=args.batch_size, workers=args.workers, ext="npy",
                                predict_freq=args.predict_freq, use_cuda=args.CUDA, input_size=1, hidden_size=args.hidden_size, num_layers=args.num_layers)
        else:
            raise ValueError("Unknown data mode %s" % args.data_mode)
    elif args.domain_mode == "meta":
        assert len(domains) - args.train_domain - args.validation_domain > 0
        modelfile = path.join(args.outdir, "model.pt")
        domain_modelfile = path.join(args.outdir, "domain_model.pt")
        if args.data_mode == "fewshot":
            from trainers_debug import UCRArchiveMetaLearnTrainer as Trainer
            trainer = Trainer()

            np.random.seed(args.domain_seed)
            train_domains = np.random.choice(
                domains, args.train_domain, replace=False)
            test_domains = np.setdiff1d(domains, train_domains)
            val_domains = np.random.choice(
                test_domains, args.validation_domain, replace=False)
            test_domains = np.setdiff1d(test_domains, val_domains)
            for data_type, target_domains, filelist in zip(["train", "validation", "query"], [train_domains, val_domains, test_domains], [trainer.trains, trainer.validations, trainer.tests]):
                for domain in target_domains:
                    basename = "%s.%s.npy" % (domain, data_type)
                    target_org_file = path.join(args.root_dir, basename)
                    support_org_file = path.join(
                        args.root_dir, "%s.support.npy" % (domain))
                    assert path.isfile(target_org_file) and path.isfile(
                        support_org_file)
                    target_dir = path.join(args.outdir, data_type)
                    target_file = path.join(target_dir, basename)
                    support_file = path.join(
                        target_dir, "%s.support.npy" % (domain))
                    if not path.isdir(target_dir):
                        os.makedirs(target_dir)
                    if not path.isfile(target_file) and not path.islink(target_file):
                        os.symlink(path.abspath(target_org_file), target_file)
                    if not path.isfile(support_file) and not path.islink(support_file):
                        os.symlink(path.abspath(
                            support_org_file), support_file)
                    # ファイルリストに入力ファイルとsupportファイルのタプルを登録する
                    # 学習時は入力ファイルのみを使用する。
                    filelist.append((target_file, support_file))

            if not path.isfile(modelfile) or not path.isfile(domain_modelfile) or args.RETRY:
                trainer.train(modelfile, domain_modelfile, epochs=args.epochs, batch_size=args.batch_size, min_delta=args.min_delta, max_datas=args.max_datas,
                              patience=args.patience, restore_best_weight=args.restore_best_weight,
                              workers=args.workers, predict_freq=args.predict_freq, use_cuda=args.CUDA, seed=args.seed,
                              input_size=1, hidden_size=args.hidden_size, num_layers=args.num_layers,
                              domain_input_size=1, domain_hidden_size=args.domain_hidden_size, domain_num_layers=args.domain_num_layers, domain_output_size=args.domain_output_size,
                              lr=args.learning_rate, reduction="sum")
            if not path.isfile(scorefile) or args.RETRY:
                trainer.predict_targets = ([trainer.trains, trainer.validations, trainer.tests], [
                                           "train", "validation", "query"])
                trainer.predict(args.outdir, modelfile, domain_modelfile, batch_size=args.batch_size, workers=args.workers, ext="npy",
                                predict_freq=args.predict_freq, use_cuda=args.CUDA,
                                input_size=1, hidden_size=args.hidden_size, num_layers=args.num_layers,
                                domain_input_size=1, domain_hidden_size=args.domain_hidden_size, domain_num_layers=args.domain_num_layers, domain_output_size=args.domain_output_size)
        else:
            raise ValueError("Unknown data mode %s" % args.data_mode)
    else:
        raise ValueError("Unknown domain mode %s" % args.domain_mode)
