#! -*- coding: utf-8
import logging
import os
import sys
from datetime import datetime
from os import path

import numpy as np

from .AbstractTrainer import AbstractTrainer
from datasets import LstmDataset as Dataset
from datasets import LinearDataset as LinearDataset

__all__ = ["LstmTrainer",
           "LinearTrainer", "NonLinearTrainer"]


class LstmTrainer(AbstractTrainer):
    def __init__(self,
                 predict_freq: int = 1,
                 num_query: int = 50,
                 num_train_support: int = 0,
                 num_predict_support: int = 0,
                 support_seq_len: int = 5,
                 query_seq_len: int = 5,
                 num_ar_feature: int = 3,
                 use_cuda: bool = False,
                 batch_size: int = 50,
                 workers: int = 1,
                 delimiter: str = "\t",
                 **kwargs):
        super(LstmTrainer, self).__init__(
            predict_freq=predict_freq,
            num_query=num_query,
            num_train_support=num_train_support,
            num_predict_support=num_predict_support,
            support_seq_len=support_seq_len,
            query_seq_len=query_seq_len,
            num_ar_feature=num_ar_feature,
            use_cuda=use_cuda,
            batch_size=batch_size,
            workers=workers,
            delimiter=delimiter,
            **kwargs)

    def Model(self, **kwargs):
        """
        input_size: int
        hidden_size: int
        num_layers: int
        """
        assert "input_size" in kwargs and "hidden_size" in kwargs and "num_layers" in kwargs
        input_size = int(kwargs["input_size"])
        hidden_size = int(kwargs["hidden_size"])
        num_layers = int(kwargs["num_layers"])

        from models import LstmModel as Model
        return Model(input_size, hidden_size, num_layers)


class LinearTrainer(AbstractTrainer):
    def __init__(self,
                 predict_freq: int = 1,
                 num_query: int = 50,
                 num_train_support: int = 0,
                 num_predict_support: int = 0,
                 support_seq_len: int = 5,
                 query_seq_len: int = 5,
                 num_ar_feature: int = 3,
                 use_cuda: bool = False,
                 batch_size: int = 50,
                 workers: int = 1,
                 delimiter: str = "\t",
                 **kwargs):
        super(LinearTrainer, self).__init__(
            predict_freq=predict_freq,
            num_query=num_query,
            num_train_support=num_train_support,
            num_predict_support=num_predict_support,
            support_seq_len=support_seq_len,
            query_seq_len=query_seq_len,
            num_ar_feature=num_ar_feature,
            use_cuda=use_cuda,
            batch_size=batch_size,
            workers=workers,
            delimiter=delimiter,
            **kwargs)

    def Model(self, **kwargs):
        """
        input_size: int
        """
        assert "input_size" in kwargs
        from models import LinearModel as Model
        input_size = self.num_ar_feature
        output_size = int(kwargs["input_size"])
        return Model(input_size, output_size)

    def Dataset(self, queryfile, supportfile,
                tag: str = "",
                num_query: int = None,
                num_support: int = 3,
                query_seq_len: int = None,
                train: bool = False,
                **kwargs):
        return LinearDataset(queryfile, supportfile,
                             tag=tag,
                             predict_freq=self.predict_freq,
                             num_query=num_query,
                             num_support=num_support,
                             support_seq_len=self.support_seq_len,
                             query_seq_len=query_seq_len,
                             num_ar_feature=self.num_ar_feature,
                             train=train)


class NonLinearTrainer(LinearTrainer):
    def __init__(self,
                 predict_freq: int = 1,
                 num_query: int = 50,
                 num_train_support: int = 0,
                 num_predict_support: int = 0,
                 support_seq_len: int = 5,
                 query_seq_len: int = 5,
                 num_ar_feature: int = 3,
                 use_cuda: bool = False,
                 batch_size: int = 50,
                 workers: int = 1,
                 delimiter: str = "\t",
                 **kwargs):
        super(NonLinearTrainer, self).__init__(
            predict_freq=predict_freq,
            num_query=num_query,
            num_train_support=num_train_support,
            num_predict_support=num_predict_support,
            support_seq_len=support_seq_len,
            query_seq_len=query_seq_len,
            num_ar_feature=num_ar_feature,
            use_cuda=use_cuda,
            batch_size=batch_size,
            workers=workers,
            delimiter=delimiter,
            **kwargs)

    def Model(self, **kwargs):
        """
        input_size: int
        hidden_size: int
        """
        assert "input_size" in kwargs and "hidden_size" in kwargs
        from models import NonLinearModel as Model
        input_size = self.num_ar_feature
        hidden_size = int(kwargs["hidden_size"])
        output_size = int(kwargs["input_size"])
        return Model(input_size, output_size, hidden_size)
