#! /bin/sh

hostname
echo $*

test_flg=$1
jobset=$2
shift 2

epochs=$1
shift

# echo $*
method="$1"
mode="$2"
dataset="$3"
seq_len=$4
data_rootdir="$5"
out_rootdir="$6"
predict_support=$7
train_support=$8
shift 8
# echo $*
extra_args="$*"

# train_domains=(55 50 40 30 20 10)
# num_queries=(50 40 30 20 10)
train_domains=(55)
num_queries=(50)
if [ "${jobset}" = "domain" ]; then
    train_domains=(50 40 30 20 10)
elif [ "${jobset}" = "data" ]; then
    num_queries=(40 30 20 10)
elif [ "${jobset}" = "data30" ]; then
    num_queries=(30)
elif [ "${jobset}" = "domain_fullset" ]; then
    train_domains=(55 50 40 30 20 10)
elif [ "${jobset}" = "data_fullset" ]; then
    num_queries=(50 40 30 20 10)
elif [ "${jobset}" = "default" ]; then
    train_domains=(55)
    num_queries=(30)
fi

if [ ${test_flg} -eq 1 ]; then
    loglevel=TRACE3
else
    loglevel=INFO
fi

batch_size=50
patience=1000
script="scripts/train.py"
lr=0.001
inner_lr=0.01
inner_iter=5
workers=1

hidden=32
num_layers=1
domain_num_layers=${num_layers}
domain_hidden_size=${hidden}
domain_output_size=16
domain_forward_unit00=64
domain_forward_unit01=32
domain_forward_unit02=16

num_ar_feature=3
support_seq_len=${seq_len}
# query_seq_len=${seq_len}
predict_freq=1


for train_domain in ${train_domains[@]}; do
    for num_query in ${num_queries[@]}; do
        # setup common script options.
        script_opts="--batch_size $batch_size"
        script_opts="${script_opts} --epochs $epochs"
        script_opts="${script_opts} --patience $patience"
        script_opts="${script_opts} --restore_best_weight"
        script_opts="${script_opts} --loglevel ${loglevel}"
        script_opts="${script_opts} --seed 7"
        script_opts="${script_opts} --workers $workers"
        script_opts="${script_opts} --predict_freq ${predict_freq}"
        script_opts="${script_opts} --num_ar_feature ${num_ar_feature}"
        script_opts="${script_opts} --learning_rate ${lr}"
        script_opts="${script_opts} --num_query ${num_query}"
        script_opts="${script_opts} --num_layers ${num_layers}"
        script_opts="${script_opts} --hidden_size ${hidden}"
        script_opts="${script_opts} --train_domain ${train_domain}"
        if [ ! -v ${support_seq_len} ]; then
            script_opts="${script_opts} --support_seq_len ${support_seq_len}"
        fi
        if [ ! -v ${query_seq_len} ]; then
            script_opts="${script_opts} --query_seq_len ${query_seq_len}"
        fi
        script_opts="${script_opts} ${extra_args}"

        script_cpu_opts="${script_opts}"
        script_gpu_opts="${script_opts}"
        # script_gpu_opts="${script_opts} --CUDA"

        datadir="${data_rootdir}/dataset${dataset}"
        outdir="${out_rootdir}/train${train_domain}_predict_support${predict_support}_data${num_query}_seq${seq_len}/dataset${dataset}/${method}"

        if [ "${method}" = "cross_domain" ]; then
            script_method_opts=""
            script_method_opts="${script_method_opts} --num_train_support 0"
            script_method_opts="${script_method_opts} --num_predict_support 0"
            command="python ${script} cross ${mode} ${datadir} ${outdir} ${script_gpu_opts} ${script_method_opts}"
        
        elif [ "${method}" = "linear" ]; then 
            script_method_opts=""
            script_method_opts="${script_method_opts} --num_train_support 0"
            script_method_opts="${script_method_opts} --num_predict_support 0"
            command="python ${script} linear ${mode} ${datadir} ${outdir} ${script_gpu_opts} ${script_method_opts}"

        elif [ "${method}" = "nonlinear" ]; then 
            script_method_opts=""
            script_method_opts="${script_method_opts} --num_train_support 0"
            script_method_opts="${script_method_opts} --num_predict_support 0"
            command="python ${script} nonlinear ${mode} ${datadir} ${outdir} ${script_gpu_opts} ${script_method_opts}"

        elif [ "${method}" = "meta_learning" ]; then 
            script_method_opts=""
            script_method_opts="${script_method_opts} --num_train_support ${train_support}"
            script_method_opts="${script_method_opts} --num_predict_support ${predict_support}"
            script_method_opts="${script_method_opts} --domain_num_layers ${domain_num_layers}"
            script_method_opts="${script_method_opts} --domain_hidden_size ${domain_hidden_size}"
            script_method_opts="${script_method_opts} --domain_output_size ${domain_output_size}"
            script_method_opts="${script_method_opts} --domain_forward_unit00 ${domain_forward_unit00}"
            script_method_opts="${script_method_opts} --domain_forward_unit01 ${domain_forward_unit01}"
            script_method_opts="${script_method_opts} --domain_forward_unit02 ${domain_forward_unit02}"
            command="python ${script} meta ${mode} ${datadir} ${outdir} ${script_gpu_opts} ${script_method_opts}"
        
        elif [ "${method}" = "bidirectional" ]; then 
            script_method_opts=""
            script_method_opts="${script_method_opts} --num_train_support ${train_support}"
            script_method_opts="${script_method_opts} --num_predict_support ${predict_support}"
            script_method_opts="${script_method_opts} --domain_num_layers ${domain_num_layers}"
            script_method_opts="${script_method_opts} --domain_hidden_size ${domain_hidden_size}"
            script_method_opts="${script_method_opts} --domain_output_size ${domain_output_size}"
            script_method_opts="${script_method_opts} --domain_forward_unit00 ${domain_forward_unit00}"
            script_method_opts="${script_method_opts} --domain_forward_unit01 ${domain_forward_unit01}"
            script_method_opts="${script_method_opts} --domain_forward_unit02 ${domain_forward_unit02}"
            command="python ${script} ${method} ${mode} ${datadir} ${outdir} ${script_gpu_opts} ${script_method_opts}"
        
        elif [ "${method}" = "attention" ]; then
            script_method_opts=""
            script_method_opts="${script_method_opts} --num_train_support ${train_support}"
            script_method_opts="${script_method_opts} --num_predict_support ${predict_support}"
            command="python ${script} ${method} ${mode} ${datadir} ${outdir} ${script_gpu_opts} ${script_method_opts}"
        
        elif [ "${method}" = "attention_only" ]; then
            script_method_opts=""
            script_method_opts="${script_method_opts} --num_train_support ${train_support}"
            script_method_opts="${script_method_opts} --num_predict_support ${predict_support}"
            command="python ${script} ${method} ${mode} ${datadir} ${outdir} ${script_gpu_opts} ${script_method_opts}"
        
        elif [ "${method}" = "maml" ]; then 
            script_method_opts=""
            script_method_opts="${script_method_opts} --num_train_support ${train_support}"
            script_method_opts="${script_method_opts} --num_predict_support ${predict_support}"
            script_method_opts="${script_method_opts} --inner_learning_rate ${inner_lr}"
            script_method_opts="${script_method_opts} --inner_iter ${inner_iter}"
            command="python ${script} ${method} ${mode} ${datadir} ${outdir} ${script_cpu_opts} ${script_method_opts}"
        
        elif [ "${method}" = "maml_linear" ]; then 
            script_method_opts=""
            script_method_opts="${script_method_opts} --num_train_support ${train_support}"
            script_method_opts="${script_method_opts} --num_predict_support ${predict_support}"
            script_method_opts="${script_method_opts} --inner_learning_rate ${inner_lr}"
            script_method_opts="${script_method_opts} --inner_iter ${inner_iter}"
            command="python ${script} ${method} ${mode} ${datadir} ${outdir} ${script_gpu_opts} ${script_method_opts}"
        
        elif [ "${method}" = "maml_nonlinear" ]; then 
            script_method_opts=""
            script_method_opts="${script_method_opts} --num_train_support ${train_support}"
            script_method_opts="${script_method_opts} --num_predict_support ${predict_support}"
            script_method_opts="${script_method_opts} --inner_learning_rate ${inner_lr}"
            script_method_opts="${script_method_opts} --inner_iter ${inner_iter}"
            command="python ${script} ${method} ${mode} ${datadir} ${outdir} ${script_gpu_opts} ${script_method_opts}"
        
        else
            echo Invalid sub command ${method}
            exit 1
        fi 
        

        if [ ! -e ${outdir} ]; then
            mkdir -p ${outdir}
        fi
        echo ${command}
        ${command}

        if [ ${test_flg} -eq 1 ]; then
            break
        fi
    done
    if [ ${test_flg} -eq 1 ]; then
        break
    fi
done

if [ ${test_flg} -eq 0 ]; then
    bash plot_cross_domain.sh ${jobset} ${method} ${dataset} ${seq_len} ${out_rootdir}
fi
